## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Sevopedia 
##   sevotastic.blogspot.com
##   sevotastic@yahoo.com
##


from CvPythonExtensions import *
import CvUtil
import ScreenInput
import CvScreenEnums
import CvPediaScreen
import string

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaBuilding( CvPediaScreen.CvPediaScreen ):
	"Civilopedia Screen for Buildings"
	
	def __init__(self, main):

		CvPediaScreen.CvPediaScreen.__init__(self)
		self.iBuilding = -1
		self.bLastBuildingType = false
		self.top = main

		self.WORLD_WONDER = 1
		self.TEAM_WONDER = 2
		self.NATIONAL_WONDER = 3

		self.BUTTON_SIZE = 46
		
		self.X_BUILDING_PANE = self.top.X_PEDIA_PAGE + 20
		self.Y_BUILDING_PANE = self.top.Y_PEDIA_PAGE + 4
		self.W_BUILDING_PANE = 323
		self.H_BUILDING_PANE = 116
		
		self.X_BUILDING_ANIMATION = self.X_BUILDING_PANE + self.W_BUILDING_PANE + 13
		self.Y_BUILDING_ANIMATION = 62
		self.W_BUILDING_ANIMATION = 1000 - self.X_BUILDING_ANIMATION
		self.H_BUILDING_ANIMATION = 218
		self.X_ROTATION_BUILDING_ANIMATION = -20
		self.Z_ROTATION_BUILDING_ANIMATION = 30
		self.SCALE_ANIMATION = 0.75
		
		self.X_ICON_PANE = self.X_BUILDING_PANE + 8
		self.Y_ICON_PANE = self.Y_BUILDING_PANE + 8
		self.W_ICON_PANE = 100
		self.H_ICON_PANE = 100
		
		self.ICON_SIZE = 64
		self.X_ICON = self.X_ICON_PANE + self.W_ICON_PANE/2 - self.ICON_SIZE/2
		self.Y_ICON = self.Y_ICON_PANE + self.H_ICON_PANE/2 - self.ICON_SIZE/2

		self.X_STATS_PANE = self.X_ICON_PANE + self.W_ICON_PANE + 8
		self.Y_STATS_PANE = self.Y_ICON - 5
		self.W_STATS_PANE = 200
		self.H_STATS_PANE = self.ICON_SIZE + 20

		self.X_PREREQ_PANE = self.X_BUILDING_PANE
		self.Y_PREREQ_PANE = 171
		self.W_PREREQ_PANE = 323
		self.H_PREREQ_PANE = 110

		self.X_SPECIAL_PANE = self.X_BUILDING_PANE
		self.Y_SPECIAL_PANE = self.Y_PREREQ_PANE+self.H_PREREQ_PANE
		self.W_SPECIAL_PANE = 1000 - self.X_SPECIAL_PANE
		self.H_SPECIAL_PANE = 190
		
		self.X_HISTORY_PANE = self.X_BUILDING_PANE
		self.Y_HISTORY_PANE = 475
		self.W_HISTORY_PANE = 1000 - self.X_SPECIAL_PANE
		self.H_HISTORY_PANE = 230
		
	# Screen construction function
	def interfaceScreen(self, iBuilding):	
			
		self.iBuilding = iBuilding
		self.info = gc.getBuildingInfo(self.iBuilding)
		
		# Original unit saved for image, animation.
		self.iShowBuilding = iBuilding
		self.showInfo = self.info
		
		if self.isHideIdentical():
			iDefaultBuilding = self.getDefaultBuilding(self.info)
			
			# Check if just linked to identical building, if so replace with default instead.
			if self.isIdentical(self.info, iBuilding):
				if iDefaultBuilding >= 0:
					self.iBuilding = iDefaultBuilding
					self.info = gc.getBuildingInfo(iDefaultBuilding)
			
			# Check if this is default building and active civ has identical building, if so use identical building for image, animation.
			elif gc.getGame().getActivePlayer() >= 0:
				civInfo = gc.getCivilizationInfo(gc.getGame().getActiveCivilizationType())
				iUniqueBuilding = civInfo.getCivilizationBuildings(self.info.getBuildingClassType())
				uniqueBuildingInfo = gc.getBuildingInfo(iUniqueBuilding)
				if iDefaultBuilding == iBuilding and uniqueBuildingInfo is not None and self.isIdentical(uniqueBuildingInfo, iUniqueBuilding):
					self.iShowBuilding = iUniqueBuilding
					self.showInfo = uniqueBuildingInfo
	
		self.top.deleteAllWidgets()						
							
		screen = self.top.getScreen()
		
		bNotActive = (not screen.isActive())
		if bNotActive:
			self.top.setPediaCommonWidgets()
		
		# Header...
		szHeader = u"<font=4b>" + self.getTitle(self.info, self.iBuilding).upper() + u"</font>"
		szHeaderId = self.top.getNextWidgetName()
		screen.setLabel(szHeaderId, "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.top.X_SCREEN, self.top.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, CivilopediaPageTypes.CIVILOPEDIA_PAGE_BUILDING, self.iBuilding)
		
		# Top
		if self.getWonderClass(iBuilding) > -1:
			link = CivilopediaPageTypes.CIVILOPEDIA_PAGE_WONDER
		else:
			link = CivilopediaPageTypes.CIVILOPEDIA_PAGE_BUILDING
		# screen.setText(self.top.getNextWidgetName(), "Background", self.top.MENU_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.top.X_MENU, self.top.Y_MENU, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_MAIN, link, -1)

		if self.top.iLastScreen	!= CvScreenEnums.PEDIA_BUILDING or bNotActive:	
			if self.top.iLastScreen != CvScreenEnums.PEDIA_MAIN:
				self.placeLinks()	
			self.top.iLastScreen = CvScreenEnums.PEDIA_BUILDING
					
		screen.addPanel( self.top.getNextWidgetName(), "", "", False, False, self.X_BUILDING_PANE, self.Y_BUILDING_PANE, self.W_BUILDING_PANE, self.H_BUILDING_PANE, PanelStyles.PANEL_STYLE_BLUE50)
		
		# Icon
		screen.addPanel(self.top.getNextWidgetName(), "", "", false, false, self.X_ICON_PANE, self.Y_ICON_PANE, self.W_ICON_PANE, self.H_ICON_PANE, PanelStyles.PANEL_STYLE_MAIN)
		screen.addDDSGFC(self.top.getNextWidgetName(), self.showInfo.getButton(), self.X_ICON, self.Y_ICON, self.ICON_SIZE, self.ICON_SIZE, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		# Building animation
		if (self.top.isShowAnimationsEnabled()):
			screen.addBuildingGraphicGFC(self.top.getNextWidgetName(), self.iShowBuilding, self.X_BUILDING_ANIMATION, self.Y_BUILDING_ANIMATION, self.W_BUILDING_ANIMATION, self.H_BUILDING_ANIMATION, WidgetTypes.WIDGET_GENERAL, -1, -1, self.X_ROTATION_BUILDING_ANIMATION, self.Z_ROTATION_BUILDING_ANIMATION, self.SCALE_ANIMATION, True)

		self.placeStats()
		
		self.placeRequires()

		self.placeSpecial()
		
		self.placeHistory()

			
	# Place happiness/health/commerce/great people modifiers
	def placeStats(self):

		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()

		screen.addListBoxGFC(panelName, "", self.X_STATS_PANE, self.Y_STATS_PANE, self.W_STATS_PANE, self.H_STATS_PANE, TableStyles.TABLE_STYLE_EMPTY)
		screen.enableSelect(panelName, False)

		if (self.info.getProductionCost() > 0):
			if self.top.iActivePlayer == -1:
				szCost = localText.getText("TXT_KEY_PEDIA_COST", ((self.info.getProductionCost() * gc.getDefineINT("PROJECT_PRODUCTION_PERCENT"))/100,))
			else:
				szCost = localText.getText("TXT_KEY_PEDIA_COST", (gc.getPlayer(self.top.iActivePlayer).getBuildingProductionNeeded(self.iBuilding),))
			screen.appendListBoxString(panelName, u"<font=3>" + szCost.upper() + u"%c" % gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar() + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)

		for k in range(YieldTypes.NUM_YIELD_TYPES):
			if (self.info.getYieldChange(k) != 0):
				if (self.info.getYieldChange(k) > 0):
					szSign = "+"
				else:
					szSign = ""
					
				szYield = gc.getYieldInfo(k).getDescription() + ": "
				
				szText1 = szYield.upper() + szSign + str(self.info.getYieldChange(k))
				szText2 = szText1 + (u"%c" % (gc.getYieldInfo(k).getChar()))
				screen.appendListBoxString(panelName, u"<font=3>" + szText2 + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)

			
		for k in range(CommerceTypes.NUM_COMMERCE_TYPES):
		
			iTotalCommerce = self.info.getObsoleteSafeCommerceChange(k) + self.info.getCommerceChange(k)
			if (iTotalCommerce != 0):
				if (iTotalCommerce > 0):
					szSign = "+"
				else:
					szSign = ""
					
				szCommerce = gc.getCommerceInfo(k).getDescription() + ": "
				
				szText1 = szCommerce.upper() + szSign + str(iTotalCommerce)
				szText2 = szText1 + (u"%c" % (gc.getCommerceInfo(k).getChar()))
				screen.appendListBoxString(panelName, u"<font=3>" + szText2 + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)

		iHappiness = self.info.getHappiness()
		if self.top.iActivePlayer != -1:
			if (self.iBuilding == gc.getCivilizationInfo(gc.getPlayer(self.top.iActivePlayer).getCivilizationType()).getCivilizationBuildings(self.info.getBuildingClassType())):
				iHappiness += gc.getPlayer(self.top.iActivePlayer).getExtraBuildingHappiness(self.iBuilding)

		if (iHappiness > 0):
			szText = localText.getText("TXT_KEY_PEDIA_HAPPY", (iHappiness,)).upper()
			screen.appendListBoxString(panelName, u"<font=3>" + szText + u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR) + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)

		elif (iHappiness < 0):
			szText = localText.getText("TXT_KEY_PEDIA_UNHAPPY", (-iHappiness,)).upper()
			screen.appendListBoxString(panelName, u"<font=3>" + szText + u"%c" % CyGame().getSymbolID(FontSymbols.UNHAPPY_CHAR) + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)

		if (self.info.getHealth() > 0):
			szText = localText.getText("TXT_KEY_PEDIA_HEALTHY", (self.info.getHealth(),)).upper()
			screen.appendListBoxString(panelName, u"<font=3>" + szText + u"%c" % CyGame().getSymbolID(FontSymbols.HEALTHY_CHAR) + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)
			
		elif (self.info.getHealth() < 0):
			szText = localText.getText("TXT_KEY_PEDIA_UNHEALTHY", (-self.info.getHealth(),)).upper()
			screen.appendListBoxString(panelName, u"<font=3>" + szText + u"%c" % CyGame().getSymbolID(FontSymbols.UNHEALTHY_CHAR) + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)
		
		if (self.info.getGreatPeopleRateChange() != 0):
			szText = localText.getText("TXT_KEY_PEDIA_GREAT_PEOPLE", (self.info.getGreatPeopleRateChange(),)).upper()
			screen.appendListBoxString(panelName, u"<font=3>" + szText + u"%c" % CyGame().getSymbolID(FontSymbols.GREAT_PEOPLE_CHAR) + u"</font>", WidgetTypes.WIDGET_GENERAL, 0, 0, CvUtil.FONT_LEFT_JUSTIFY)
			
	# Place prereqs (techs, resources)
	def placeRequires(self):

		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_PEDIA_REQUIRES", ()), "", false, true,
                                 self.X_PREREQ_PANE, self.Y_PREREQ_PANE, self.W_PREREQ_PANE, self.H_PREREQ_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
						
		screen.attachLabel(panelName, "", "  ")

		# add tech buttons
		for iPrereq in range(gc.getNumTechInfos()):
			if isTechRequiredForBuilding(iPrereq, self.iBuilding):
				screen.attachImageButton( panelName, "", gc.getTechInfo(iPrereq).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_TECH, iPrereq, 1, False )

		# add resource buttons
		iPrereq = self.info.getPrereqAndBonus()
		if (iPrereq >= 0):
			screen.attachImageButton( panelName, "", gc.getBonusInfo(iPrereq).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BONUS, iPrereq, -1, False )
			
		for k in range(gc.getNUM_BUILDING_PREREQ_OR_BONUSES()):
			iPrereq = self.info.getPrereqOrBonuses(k)
			if (iPrereq >= 0):
				screen.attachImageButton( panelName, "", gc.getBonusInfo(iPrereq).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BONUS, iPrereq, -1, False )
				
		# add religion button
		iPrereq = self.info.getPrereqReligion()
		if (iPrereq >= 0):
			screen.attachImageButton( panelName, "", gc.getReligionInfo(iPrereq).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_RELIGION, iPrereq, -1, False )
		
	# Place Special abilities
	def placeSpecial(self):

		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_PEDIA_SPECIAL_ABILITIES", ()), "", true, false,
                                 self.X_SPECIAL_PANE, self.Y_SPECIAL_PANE, self.W_SPECIAL_PANE, self.H_SPECIAL_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		
		listName = self.top.getNextWidgetName()
		
		szSpecialText = CyGameTextMgr().getBuildingHelp(self.iBuilding, True, False, False, None)[1:]
		
		iDefaultBuilding = self.getDefaultBuilding(self.info)
		
		# No Default Building Fix
		if iDefaultBuilding == -1:
			for iCiv in range(gc.getNumCivilizationInfos()):
				civInfo = gc.getCivilizationInfo(iCiv)
				if civInfo.getCivilizationBuildings(self.info.getBuildingClassType()) == self.iBuilding:
					szUnique = localText.getText("TXT_KEY_UNIQUE_BUILDING", (civInfo.getTextKey(), ))
					if szSpecialText.find(szUnique) == -1:
						szSpecialText = szUnique + "\n" + szSpecialText
		
		szSpecialText = self.getWonderClassString() + szSpecialText

		# Replaced by Buildings
		if iDefaultBuilding == self.iBuilding:
			for k in range(gc.getNumBuildingInfos()):
				kBuildingInfo = gc.getBuildingInfo(k)
				if (k != self.iBuilding) and (self.getDefaultBuilding(kBuildingInfo) == self.iBuilding) and (not self.isFiltered(kBuildingInfo, k)):
					if szSpecialText != "":
						szSpecialText += "\n"
					szSpecialText += localText.getText("TXT_KEY_REPLACED_BY_UNIT", (kBuildingInfo.getDescription(), ))

		screen.addMultilineText(listName, szSpecialText, self.X_SPECIAL_PANE+5, self.Y_SPECIAL_PANE+30, self.W_SPECIAL_PANE-10, self.H_SPECIAL_PANE-35, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)


	def placeHistory(self):
		
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, "History", "", True, True,
						self.X_HISTORY_PANE, self.Y_HISTORY_PANE,
						self.W_HISTORY_PANE, self.H_HISTORY_PANE,
						PanelStyles.PANEL_STYLE_BLUE50 )
		
		textName = self.top.getNextWidgetName()
#		screen.attachMultilineText( panelName, textName, self.info.getCivilopedia(),WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
		screen.addMultilineText( textName, self.info.getCivilopedia(), self.X_HISTORY_PANE + 15, self.Y_HISTORY_PANE + 40,
		    self.W_HISTORY_PANE - (15 * 2), self.H_HISTORY_PANE - (15 * 2) - 25, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
		

	def placeLinks(self):
		
		screen = self.top.getScreen()

		self.top.placeLinks(CivilopediaPageTypes.CIVILOPEDIA_PAGE_BUILDING)

		if self.getWonderClass(self.iBuilding) >= 0:
			self.top.placeWonders(self.iBuilding)
		else:
			self.top.placeBuildings(self.iBuilding)


	def getWonderClassString(self):
		iBuildingClass = self.info.getBuildingClassType()
		szBuildingType = u"%c" %(CyGame().getSymbolID(FontSymbols.BULLET_CHAR))
                
		if (isWorldWonderClass(iBuildingClass)):
			iMaxInstances = gc.getBuildingClassInfo(iBuildingClass).getMaxGlobalInstances()
			szBuildingType += localText.getText("TXT_KEY_PEDIA_WORLD_WONDER", ())
			if (iMaxInstances > 1):
				szBuildingType += " " + localText.getText("TXT_KEY_PEDIA_WONDER_INSTANCES", (iMaxInstances,))
			return szBuildingType + "\n"

		if (isTeamWonderClass(iBuildingClass)):
			iMaxInstances = gc.getBuildingClassInfo(iBuildingClass).getMaxTeamInstances()
			szBuildingType += localText.getText("TXT_KEY_PEDIA_TEAM_WONDER", ())
			if (iMaxInstances > 1):
				szBuildingType += " " + localText.getText("TXT_KEY_PEDIA_WONDER_INSTANCES", (iMaxInstances,))
			return szBuildingType + "\n"

		if (isNationalWonderClass(iBuildingClass)):
			iMaxInstances = gc.getBuildingClassInfo(iBuildingClass).getMaxPlayerInstances()
			szBuildingType += localText.getText("TXT_KEY_PEDIA_NATIONAL_WONDER", ())
			if (iMaxInstances > 1):
				szBuildingType += " " + localText.getText("TXT_KEY_PEDIA_WONDER_INSTANCES", (iMaxInstances,))
			return szBuildingType + "\n"
		
		return ""
		
	def getWonderClass(self, iBuilding):
		iClass = gc.getBuildingInfo(iBuilding).getBuildingClassType()
		
		if isWorldWonderClass(iClass):
			return self.WORLD_WONDER
		elif isTeamWonderClass(iClass):
			return self.TEAM_WONDER
		elif isNationalWonderClass(iClass):
			return self.NATIONAL_WONDER
		return -1
		
	def getBuildingSortedList(self, bWonder):
		listBuildings = []
		iCount = 0
		for iBuilding in range(gc.getNumBuildingInfos()):
			if ((self.getWonderClass(iBuilding) > -1) == bWonder and not self.isFiltered(gc.getBuildingInfo(iBuilding), iBuilding)):
				listBuildings.append(iBuilding)
				iCount += 1
		
		listSorted = [(0,0)] * iCount
		iI = 0
		for iBuilding in listBuildings:
			listSorted[iI] = (self.getTitle(gc.getBuildingInfo(iBuilding), iBuilding), iBuilding)
			iI += 1
		listSorted.sort()
		return listSorted

	def isIdentical(self, info, iIndex):
		' Determine if this is identical. Overridden for unique units/buildings '
		
		# If building is unique...
		iDefaultBuilding = self.getDefaultBuilding(info)
		if iDefaultBuilding != iIndex and iDefaultBuilding >= 0:
			
			# Is it identical?
			defaultBuildingInfo = gc.getBuildingInfo(iDefaultBuilding)
			return defaultBuildingInfo.getTextKey() == info.getTextKey()
		
		return False

	def getTitle( self, info, iBuilding ):
		' What to use as the title. Overriden for unique units/buildings '

		if self.isShowCivNameForUnique():

			# If building is unique...
			iDefaultBuilding = self.getDefaultBuilding(info)
			if iDefaultBuilding != iBuilding:

				# get Civilization name
				civName = ""
				iCount = 0
				for iCiv in range(gc.getNumCivilizationInfos()):
					civInfo = gc.getCivilizationInfo(iCiv)
					if civInfo.getCivilizationBuildings(info.getBuildingClassType()) == iBuilding:
						iCount += 1
						civName = civInfo.getShortDescription(0)

				# Is really unique to one civ?
				if iCount == 1:
					return info.getDescription() + " (" + civName + ")"

		return info.getDescription()
	
	def getDefaultBuilding(self, info):
		buildingClassInfo = gc.getBuildingClassInfo(info.getBuildingClassType())
		if buildingClassInfo is not None:
			return buildingClassInfo.getDefaultBuildingIndex()
		return -1
											
	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		return 0

