## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Sevopedia
##   sevotastic.blogspot.com
##   sevotastic@yahoo.com
##


from CvPythonExtensions import *
import CvUtil
import ScreenInput
import CvScreenEnums
import CvPediaScreen

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaCivilization( CvPediaScreen.CvPediaScreen ):
	"Civilopedia Screen for Civilizations"

	def __init__(self, main):

		CvPediaScreen.CvPediaScreen.__init__(self)
		self.iCivilization = -1
		self.top = main
		
		self.X_MAIN_PANE = self.top.X_PEDIA_PAGE + 20
		self.Y_MAIN_PANE = self.top.Y_PEDIA_PAGE + 10
		self.W_MAIN_PANE = 200
		
		self.MAX_WIDTH = 1000
		self.MAX_HEIGHT = 700

		self.X_ICON = self.X_MAIN_PANE + 25
		self.W_ICON = 150
		self.H_ICON = 150
		self.ICON_SIZE = 80

		self.X_UNIT = self.X_MAIN_PANE + self.W_MAIN_PANE + 10
		self.Y_UNIT = 65
		self.W_UNIT = self.MAX_WIDTH - self.X_UNIT
		self.H_UNIT = 110

		self.X_BUILDING = self.X_UNIT
		self.Y_BUILDING = self.Y_UNIT + self.H_UNIT
		self.W_BUILDING = self.W_UNIT
		self.H_BUILDING = self.H_UNIT

		self.W_SEP = 20
		self.X_LEADER = self.X_MAIN_PANE
		self.Y_LEADER = self.Y_BUILDING + self.H_BUILDING
		self.W_LEADER = (self.MAX_WIDTH - self.X_LEADER - self.W_SEP) / 2
		self.H_LEADER = self.H_UNIT

		self.X_TECH = self.X_LEADER + self.W_LEADER + self.W_SEP
		self.Y_TECH = self.Y_LEADER
		self.W_TECH = self.W_LEADER
		self.H_TECH = self.H_LEADER

		self.X_TEXT = self.X_MAIN_PANE
		self.Y_TEXT = self.Y_LEADER + self.H_LEADER + 4
		self.W_TEXT = self.MAX_WIDTH - self.X_TEXT
		self.H_TEXT = self.MAX_HEIGHT - self.Y_TEXT

		self.H_MAIN_PANE = (self.Y_BUILDING + self.H_BUILDING) - self.Y_MAIN_PANE
		self.Y_ICON = self.Y_MAIN_PANE + (self.H_MAIN_PANE - self.H_ICON)/2
		
	# Screen construction function
	def interfaceScreen(self, iCivilization):	
			
		self.iCivilization = iCivilization
		self.info = gc.getCivilizationInfo(self.iCivilization)
	
		self.top.deleteAllWidgets()						
							
		screen = self.top.getScreen()
		
		bNotActive = (not screen.isActive())
		if bNotActive:
			self.top.setPediaCommonWidgets()

		# Header...
		szHeader = u"<font=4b>" + self.info.getDescription().upper() + u"</font>"
		szHeaderId = self.top.getNextWidgetName()
		screen.setLabel(szHeaderId, "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.top.X_SCREEN, self.top.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		
		# Top
		# screen.setText(self.top.getNextWidgetName(), "Background", self.top.MENU_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.top.X_MENU, self.top.Y_MENU, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_MAIN, CivilopediaPageTypes.CIVILOPEDIA_PAGE_CIV, -1)

		if self.top.iLastScreen	!= CvScreenEnums.PEDIA_CIVILIZATION or bNotActive:	
			if self.top.iLastScreen != CvScreenEnums.PEDIA_MAIN:
				self.placeLinks()	
			self.top.iLastScreen = CvScreenEnums.PEDIA_CIVILIZATION
			
		# Icon
		screen.addPanel( self.top.getNextWidgetName(), "", "", False, False,
		    self.X_MAIN_PANE, self.Y_MAIN_PANE, self.W_MAIN_PANE, self.H_MAIN_PANE, PanelStyles.PANEL_STYLE_BLUE50)
		screen.addPanel(self.top.getNextWidgetName(), "", "", false, false,
		    self.X_ICON, self.Y_ICON, self.W_ICON, self.H_ICON, PanelStyles.PANEL_STYLE_MAIN)
		screen.addDDSGFC(self.top.getNextWidgetName(), ArtFileMgr.getCivilizationArtInfo(self.info.getArtDefineTag()).getButton(),
		    self.X_ICON + self.W_ICON/2 - self.ICON_SIZE/2, self.Y_ICON + self.H_ICON/2 - self.ICON_SIZE/2, self.ICON_SIZE, self.ICON_SIZE, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		self.placeTech()
		self.placeBuilding()
		self.placeUnit()
		self.placeLeader()
		self.placeText()

		return

	def placeTech(self):
		
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_FREE_TECHS", ()), "", false, true,
				 self.X_TECH, self.Y_TECH, self.W_TECH, self.H_TECH, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
		
		for iTech in range(gc.getNumTechInfos()):
			if (self.info.isCivilizationFreeTechs(iTech)):
				screen.attachImageButton( panelName, "", gc.getTechInfo(iTech).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_TECH, iTech, 1, False )
	
	def placeBuilding(self):
		
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_UNIQUE_BUILDINGS", ()), "", false, true,
				self.X_BUILDING, self.Y_BUILDING, self.W_BUILDING, self.H_BUILDING, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
		
		for iBuilding in range(gc.getNumBuildingClassInfos()):
			iUniqueBuilding = self.info.getCivilizationBuildings(iBuilding);
			iDefaultBuilding = gc.getBuildingClassInfo(iBuilding).getDefaultBuildingIndex();
			if (iUniqueBuilding > -1 and iDefaultBuilding != iUniqueBuilding):
				uniqueBuildingInfo = gc.getBuildingInfo(iUniqueBuilding)
				if not self.top.pediaBuildingScreen.isFiltered(uniqueBuildingInfo, iUniqueBuilding):
					screen.attachImageButton( panelName, "", uniqueBuildingInfo.getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, iUniqueBuilding, 1, False )
				
	def placeUnit(self):
		
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_FREE_UNITS", ()), "", false, true,
				 self.X_UNIT, self.Y_UNIT, self.W_UNIT, self.H_UNIT, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
					
		for iUnit in range(gc.getNumUnitClassInfos()):
			iUniqueUnit = self.info.getCivilizationUnits(iUnit);
			iDefaultUnit = gc.getUnitClassInfo(iUnit).getDefaultUnitIndex();
			if (iUniqueUnit > -1 and iDefaultUnit != iUniqueUnit):
				uniqueUnitInfo = gc.getUnitInfo(iUniqueUnit)
				if not self.top.pediaUnitScreen.isFiltered(uniqueUnitInfo, iUniqueUnit):
					screen.attachImageButton( panelName, "", uniqueUnitInfo.getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT, iUniqueUnit, 1, False )
		
	def placeLeader(self):
		
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_CONCEPT_LEADERS", ()), "", false, true,
				 self.X_LEADER, self.Y_LEADER, self.W_LEADER, self.H_LEADER, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")

		for iLeader in range(gc.getNumLeaderHeadInfos()):
			civ = self.info
			if civ.isLeaders(iLeader):
				screen.attachImageButton( panelName, "", gc.getLeaderHeadInfo(iLeader).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_LEADER, iLeader, 1, False )
		
	def placeText(self):
		
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, "", "", true, true,
				 self.X_TEXT, self.Y_TEXT, self.W_TEXT, self.H_TEXT, PanelStyles.PANEL_STYLE_BLUE50 )
 
		textPane = self.top.getNextWidgetName()
		szText = self.info.getCivilopedia()
		screen.addMultilineText( textPane, szText, 
		                         self.X_TEXT+5, self.Y_TEXT+10, self.W_TEXT-10, self.H_TEXT-15,
		                         WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
													
	def placeLinks(self):

		self.top.placeLinks(CivilopediaPageTypes.CIVILOPEDIA_PAGE_CIV)
		self.top.placeCivs(self.iCivilization)


	def isGraphicalOnly( self, info, i ):
		' determine whether this is graphical only '

		return info.isGraphicalOnly() or not info.isPlayable()
			

	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		return 0


