## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
## 
## This file is a refactoring of the Civilopedia Index by Fitchn.
## I decided that all of this did not belong in CvPediaMain.py
## File created by Gaurav on 1/3/07.
##


from CvPythonExtensions import *

import string

import CvUtil
import CvScreenEnums

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaIndex:
	"Fitchn's Civilopedia Index"

	def __init__(self, main):

		self.top = main


	def placeIndex(self, iSubCategory):

		screen = self.top.getScreen()
		
		# Header...
		#szHeader = u"<font=4b>INDEX</font>"
		szHeader = u"<font=4b>" +localText.getText("TXT_KEY_PEDIA_INDEX", ()).upper() + u"</font>"
		screen.setText(self.top.getNextWidgetName(), "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.top.X_SCREEN, self.top.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_DESCRIPTION, CivilopediaPageTypes.CIVILOPEDIA_PAGE_INDEX, -1)
		screen.setImageButton(self.top.getNextWidgetName(), ArtFileMgr.getInterfaceArtInfo("INTERFACE_GENERAL_CIVILOPEDIA_ICON").getPath(), self.top.X_EXIT, self.top.Y_TITLE, 32, 32, WidgetTypes.WIDGET_PEDIA_DESCRIPTION,  CivilopediaPageTypes.CIVILOPEDIA_PAGE_INDEX, -1)

		screen.addListBoxGFC(self.top.SUBLIST_ID, "", self.top.X_ITEMS_PANE, self.top.Y_ITEMS_PANE, self.top.W_ITEMS_PANE, self.top.H_ITEMS_PANE, TableStyles.TABLE_STYLE_STANDARD)
		screen.enableSelect(self.top.SUBLIST_ID, True)
		screen.setStyle(self.top.SUBLIST_ID, "Table_StandardCiv_Style")
		
		# Create and place index pane	
		
		techList = self.top.pediaTechScreen.getSortedList( gc.getNumTechInfos(), gc.getTechInfo )
		unitList = self.top.pediaUnitScreen.getSortedList( gc.getNumUnitInfos(), gc.getUnitInfo )
		buildingList = self.top.pediaBuildingScreen.getBuildingSortedList(false) # Buildings
		wonderList = self.top.pediaBuildingScreen.getBuildingSortedList(true) # Wonders
		bonusList = self.top.pediaBonus.getSortedList( gc.getNumBonusInfos(), gc.getBonusInfo )
		improvementList = self.top.pediaImprovement.getSortedList( gc.getNumImprovementInfos(), gc.getImprovementInfo )
		promotionList = self.top.pediaPromotionScreen.getSortedList( gc.getNumPromotionInfos(), gc.getPromotionInfo )
		unitCombatList = self.top.pediaUnitChart.getSortedList( gc.getNumUnitCombatInfos(), gc.getUnitCombatInfo )
		civList = self.top.pediaCivilization.getSortedList( gc.getNumCivilizationInfos(), gc.getCivilizationInfo )
		leaderList = self.top.pediaLeader.getSortedList( gc.getNumLeaderHeadInfos(), gc.getLeaderHeadInfo )
		religionList = self.top.pediaReligion.getSortedList( gc.getNumReligionInfos(), gc.getReligionInfo )
		civicList = self.top.pediaCivic.getSortedList( gc.getNumCivicInfos(), gc.getCivicInfo )
		projectList = self.top.pediaProjectScreen.getProjectSortedList()
		terrainList = self.top.pediaTerrain.getSortedList( gc.getNumTerrainInfos(), gc.getTerrainInfo )
		featureList = self.top.pediaFeature.getSortedList( gc.getNumFeatureInfos(), gc.getFeatureInfo )
		conceptList = self.top.pediaHistorical.getSortedList( gc.getNumConceptInfos(), gc.getConceptInfo )
		specialistList = self.top.pediaSpecialist.getSortedList( gc.getNumSpecialistInfos(), gc.getSpecialistInfo )

		list=[]
		for item in techList:
			if (item[0][0:4]=="The "):
				list.append([item[0][4:]+","+item[0][0:3],"Tech",item])
			else:
				list.append([item[0],"Tech",item])
		for item in unitList:
			if (item[0][:13]=="TXT_KEY_UNIT_"):
				list.append([item[0][13:].capitalize(),"Unit",item])
			else:
				list.append([item[0],"Unit",item])
		for item in buildingList:
			if (item[0][:17]=="TXT_KEY_BUILDING_"):
				list.append([item[0][17:].capitalize(),"Building",item])
			else:
				list.append([item[0],"Building",item])
		for item in wonderList:
			if (item[0][0:4]=="The "):
				list.append([item[0][4:]+","+item[0][0:3],"Wonder",item])
			elif (item[0][:17]=="TXT_KEY_BUILDING_"):
				list.append([item[0][17:].capitalize(),"Wonder",item])
			else:
				list.append([item[0],"Wonder",item])
		for item in bonusList:
			list.append([item[0],"Bonus",item])
		for item in improvementList:
			list.append([item[0],"Improv",item])
		for item in promotionList:
			if (item[0][:18]=="TXT_KEY_PROMOTION_"):
				list.append([item[0][18:].capitalize(),"Promo",item])
			else:
				list.append([item[0],"Promo",item])
		for item in unitCombatList:
			list.append([item[0],"UnitCombat",item])
		for item in civList:
			list.append([item[0],"Civ",item])
		for item in leaderList:
			list.append([item[0],"Leader",item])
		for item in religionList:
			list.append([item[0],"Religion",item])
		for item in civicList:
			if (item[0][:14]=="TXT_KEY_CIVIC_"):
				list.append([item[0][14:].capitalize(),"Civic",item])
			else:
				list.append([item[0],"Civic",item])
		for item in projectList:
			if (item[0][0:4]=="The "):
				list.append([item[0][4:]+","+item[0][0:3],"Project",item])
			else:
				list.append([item[0],"Project",item])
		for item in terrainList:
			list.append([item[0],"Terrain",item])
		for item in featureList:
			list.append([item[0],"Feature",item])
		for item in conceptList:
			list.append([item[0],"Concept",item])
		for item in specialistList:
			if (item[0][:19]=="TXT_KEY_SPECIALIST_"):
				list.append([item[0][19:].capitalize(),"Specialist",item])
			else:
				list.append([item[0],"Specialist",item])

		list.sort()
		
		letters = []
		sLetter = "#"
		link = -1
		i = 0
		for masterItem in list:
			if (masterItem[0][:1] != sLetter):
				sLetter = masterItem[0][0:1]
				letters.append(sLetter)
				screen.appendListBoxString(self.top.SUBLIST_ID, u"<font=3>" + sLetter + u"</font>", WidgetTypes.WIDGET_PEDIA_MAIN, i + self.top.INDEX_FLAG, 0, CvUtil.FONT_LEFT_JUSTIFY )
				if i + self.top.INDEX_FLAG == iSubCategory:
					link = i
				i += 1
		
		if (link != -1):
			screen.setSelectedListBoxStringGFC(self.top.SUBLIST_ID, link)
			sLetter = letters[link]
		else:
			if self.top.isSkipBlankPage():
				self.top.pediaJump(CvScreenEnums.PEDIA_MAIN, self.top.INDEX_FLAG, False, False)
			return

		nColumns = 2
		tableName = self.top.getNextWidgetName()
		screen.addTableControlGFC(tableName, nColumns, self.top.X_PEDIA_PAGE, self.top.Y_PEDIA_PAGE, self.top.W_PEDIA_PAGE, self.top.H_PEDIA_PAGE, False, False, self.top.LIST_BUTTON_SIZE, self.top.LIST_BUTTON_SIZE, TableStyles.TABLE_STYLE_STANDARD);
		screen.enableSelect(tableName, False)
		for i in range(nColumns):
			screen.setTableColumnHeader(tableName, i, "", self.top.W_PEDIA_PAGE/nColumns-3)
		
		iCounter = 0
		screen.appendTableRow(tableName)
		for iColumn in range(nColumns):
			screen.setTableText(tableName, iColumn, 0, u"<font=4>- " + sLetter + u" -</font>", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_CENTER_JUSTIFY)
		screen.appendTableRow(tableName)
		iNumRows = 2
		iSpot = 0

		for masterItem in list:
			if (masterItem[0][:1] != sLetter):
				continue
			iSpot = iSpot+1

			if (iSpot > nColumns):
				iNumRows += 1
				screen.appendTableRow(tableName)
				iSpot = 1

			item = masterItem[2]
			iRow = iNumRows - 1
			iColumn = iSpot - 1
			if (masterItem[1]=="Tech"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getTechInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_TECH, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Unit"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getUnitInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Building"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getBuildingInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Wonder"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getBuildingInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Bonus"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getBonusInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_BONUS, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Improv"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getImprovementInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_IMPROVEMENT, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Promo"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getPromotionInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROMOTION, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="UnitCombat"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getUnitCombatInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT_COMBAT, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Civ"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getCivilizationInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIV, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Leader"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getLeaderHeadInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_LEADER, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Religion"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getReligionInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_RELIGION, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Civic"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getCivicInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIVIC, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Project"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getProjectInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROJECT, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Terrain"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getTerrainInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_TERRAIN, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Feature"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getFeatureInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_FEATURE, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Concept"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getConceptInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_DESCRIPTION, CivilopediaPageTypes.CIVILOPEDIA_PAGE_CONCEPT, item[1], CvUtil.FONT_LEFT_JUSTIFY)
			elif (masterItem[1]=="Specialist"):
				screen.setTableText(tableName, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", gc.getSpecialistInfo(item[1]).getButton(), WidgetTypes.WIDGET_PEDIA_JUMP_TO_SPECIALIST, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			iCounter += 1