## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Sevopedia ver 2.11 by Gaurav
##
## Sevopedia ver 1.93 by Sevo
##   sevotastic.blogspot.com
##   sevotastic@yahoo.com
##
## This is a complete rewrite of the Civilopedia.
## There have been changes throughout the code!
##


from CvPythonExtensions import *
import PyHelpers
import string
import CvUtil
import ScreenInput

PyPlayer = PyHelpers.PyPlayer
PyCity = PyHelpers.PyCity

import CvScreenEnums
import CvPediaScreen		# base class
import CvPediaTech
import CvPediaUnit
import CvPediaBuilding
import CvPediaPromotion
import CvPediaUnitChart
import CvPediaBonus
import CvPediaTerrain
import CvPediaFeature
import CvPediaImprovement
import CvPediaCivic
import CvPediaCivilization
import CvPediaLeader
import CvPediaSpecialist
import CvPediaHistory
import CvPediaProject
import CvPediaReligion
import UnitUpgradesGraph
import CvPediaIndex
import MoreUtil

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaMain( CvPediaScreen.CvPediaScreen ):
	"Civilopedia Main Screen"

	def __init__(self):

		CvPediaScreen.CvPediaScreen.__init__(self)
	
		self.PEDIA_MAIN_SCREEN_NAME = "PediaMainScreen"
		self.INTERFACE_ART_INFO = "SCREEN_BG_OPAQUE"

		self.WIDGET_ID = "PediaMainWidget"
		self.EXIT_ID = "PediaMainExitWidget"
		self.BACKGROUND_ID = "PediaMainBackground"
		self.TOP_PANEL_ID = "PediaMainTopPanel"
		self.BOTTOM_PANEL_ID = "PediaMainBottomPanel"
		self.BACK_ID = "PediaMainBack"
		self.NEXT_ID = "PediaMainForward"
		self.TOP_ID = "PediaMainTop"
		self.LIST_ID = "PediaMainList"
		self.SUBLIST_ID = "PediaSubList"
		self.UPGRADES_LIST = ""			# Make it accessible
		self.PROMOTIONS_LIST = ""
		self.linkList = []

		self.X_SCREEN = 500
		self.Y_SCREEN = 396
		self.W_SCREEN = 1024
		self.H_SCREEN = 768
		self.Y_TITLE = 8
		self.DY_TEXT = 45
		
		self.X_EXIT = 925
		self.Y_EXIT = 730
						
		self.X_BACK = 700
		self.Y_BACK = 10

		self.X_FORWARD = 790
		self.Y_FORWARD = 10
		
		#self.X_MENU = 450
		#self.Y_MENU = 1730

		self.BUTTON_SIZE = 64
		self.BUTTON_COLUMNS = 9

		self.LIST_BUTTON_SIZE=24
		self.INDEX_FLAG = 1024
		self.REFRESH_FLAG = 2048


		# RESIZE AND ADD ANOTHER PANEL TO THESE BLOCKS...

		self.ITEMS_MARGIN = 18
		self.ITEMS_SEPARATION = 2

		self.X_LINKS = 0 
		self.Y_LINKS = 51
		self.H_LINKS = 665
		self.W_LINKS = 175

		self.X_ITEMS_PANE = self.X_LINKS + self.W_LINKS +2
		self.Y_ITEMS_PANE = self.Y_LINKS
		self.H_ITEMS_PANE = self.H_LINKS
		self.W_ITEMS_PANE = 203

		self.X_PEDIA_PAGE = self.X_ITEMS_PANE + self.W_ITEMS_PANE
		self.Y_PEDIA_PAGE = self.Y_LINKS
		self.W_PEDIA_PAGE = 1024 - self.X_PEDIA_PAGE
		self.H_PEDIA_PAGE = self.H_LINKS
		
		self.nWidgetCount = 0
		
		# screen instances
		self.pediaTechScreen = CvPediaTech.CvPediaTech(self)
		self.pediaUnitScreen = CvPediaUnit.CvPediaUnit(self)
		self.pediaBuildingScreen = CvPediaBuilding.CvPediaBuilding(self)
		self.pediaPromotionScreen = CvPediaPromotion.CvPediaPromotion(self)
		self.pediaUnitChart = CvPediaUnitChart.CvPediaUnitChart(self)
		self.pediaBonus = CvPediaBonus.CvPediaBonus(self)
		self.pediaTerrain = CvPediaTerrain.CvPediaTerrain(self)
		self.pediaFeature = CvPediaFeature.CvPediaFeature(self)
		self.pediaImprovement = CvPediaImprovement.CvPediaImprovement(self)
		self.pediaCivic = CvPediaCivic.CvPediaCivic(self)
		self.pediaCivilization = CvPediaCivilization.CvPediaCivilization(self)
		self.pediaLeader = CvPediaLeader.CvPediaLeader(self)
		self.pediaSpecialist = CvPediaSpecialist.CvPediaSpecialist(self)
		self.pediaProjectScreen = CvPediaProject.CvPediaProject(self)
		self.pediaReligion = CvPediaReligion.CvPediaReligion(self)
		self.pediaHistorical = CvPediaHistory.CvPediaHistory(self)
		self.pediaIndex = CvPediaIndex.CvPediaIndex(self)
				
		# used for navigating "forward" and "back" in civilopedia
		self.pediaHistory = []
		self.pediaFuture = []
		
		self.listCategories = []

		self.iCategory = -1
		self.iLastScreen = -1
		self.iActivePlayer = -1
								
		self.mapCategories = { 
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_TECH	: self.placeTechs, 
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_UNIT	: self.placeUnits, 
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_BUILDING	: self.placeBuildings, 
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_WONDER	: self.placeWonders,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_TERRAIN	: self.placeTerrains,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_FEATURE	: self.placeFeatures,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_BONUS	: self.placeBoni, 
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_IMPROVEMENT	: self.placeImprovements,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_SPECIALIST	: self.placeSpecialists,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_PROMOTION	: self.placePromotions,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_UNIT_GROUP	: self.placeUnitGroups,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_CIV	: self.placeCivs,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_LEADER	: self.placeLeaders,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_RELIGION	: self.placeReligions,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_CIVIC	: self.placeCivics,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_PROJECT	: self.placeProjects,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_CONCEPT	: self.placeConcepts,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_HINTS	: self.placeHints,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_UPGRADES : self.placeUpgrades,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_PROMOTION_TREE : self.placePromotionTree,
			CivilopediaPageTypes.CIVILOPEDIA_PAGE_INDEX	: self.placeIndex,
		}


	def createDictionaries(self):

		# This function cannot be called from __init__ because the gc stuff used
		# in linkListGraphics is not available until after all the inits are called.

		# Create a hash (well, a list of lists)
		# Items are: New page order, Original List Order (& therefore Pedia Page Index), graphic
		# Painful, but I see no other way.
		self.linkList = [
			[0,0,"TECH"],
			[1,1,"STRENGTH"],
			[2,10,"STRENGTH"],
			[3,18,"MOVES"],
			[4,9,"SILVERSTAR"],
			[5,19,"SILVERSTAR"],
			[6,2,"HAMMER"],
			[7,3,"GOLDENAGE"],
			[8,15,"IRON"],
			[9,4,"MARBLE"],
			[10,5,"MARBLE"],
			[11,6,"IVORY"],
			[12,7,"FOOD"],
			[13,11,"CULTURE"],
			[14,12,"FIST"],
			[15,13,"RELIGION"],
			[16,14,"OPEN_BORDERS"],
			[17,8,"GREATPEOPLE"],
			[18,16,"MAP"],
			[19,17,"GOLD"],
			[20,20,"COMMERCE"],
		]

		self.linkListGraphics = { 
				"TECH"		: u"%c  " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar()),
				"STRENGTH"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR)),
				"HAMMER"	: u"%c  " %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()),
				"CULTURE"	: u"%c  " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar()),
				"FOOD"		: u"%c  " %(gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar()),
				"COMMERCE"	: u"%c  " %(gc.getYieldInfo(YieldTypes.YIELD_COMMERCE).getChar()),
				"GOLD"		: u"%c  " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar()),
				"GREATPEOPLE"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.GREAT_PEOPLE_CHAR)),
				"RELIGION"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.RELIGION_CHAR)),
				"IVORY"		: u"%c  " %(MoreUtil.getInfo('bonus', "BONUS_IVORY").getChar()),
				"MARBLE"	: u"%c  " %(MoreUtil.getInfo('bonus', "BONUS_MARBLE").getChar()),
				"IRON"		: u"%c  " %(MoreUtil.getInfo('bonus', "BONUS_IRON").getChar()),
				"GOLDENAGE"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.GOLDEN_AGE_CHAR)),
				"STAR"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.STAR_CHAR)),
				"SILVERSTAR"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.SILVER_STAR_CHAR)),
				"MOVES"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.MOVES_CHAR)),
				"HAPPY"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.HAPPY_CHAR)),
				"HEALTHY"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.HEALTHY_CHAR)),
				"TRADE"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.TRADE_CHAR)),
				"POWER"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.POWER_CHAR)),
				"OPEN"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.OPEN_BORDERS_CHAR)),
				"FIST"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.OCCUPATION_CHAR)),
				"HEALTH"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.HEALTHY_CHAR)),
				"MAP"		: u"%c  " %(CyGame().getSymbolID(FontSymbols.MAP_CHAR)),
				"OPEN_BORDERS"	: u"%c  " %(CyGame().getSymbolID(FontSymbols.OPEN_BORDERS_CHAR)),
				#"BUDDHISM_HOLY_CITY" : u"%c  " %(MoreUtil.getInfo('religion', "RELIGION_BUDDHISM").getHolyCityChar()),
		}


	def getScreen(self):
		return CyGInterfaceScreen(self.PEDIA_MAIN_SCREEN_NAME, CvScreenEnums.PEDIA_MAIN)
		
	def setPediaCommonWidgets(self):
		CvUtil.pyPrint("setPediaCommonWidgets()")
		self.EXIT_TEXT = u"<font=4>" + localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ()).upper() + "</font>"
		self.BACK_TEXT = u"<font=4>" + localText.getText("TXT_KEY_PEDIA_SCREEN_BACK", ()).upper() + "</font>"
		self.FORWARD_TEXT = u"<font=4>" + localText.getText("TXT_KEY_PEDIA_SCREEN_FORWARD", ()).upper() + "</font>"
		#self.MENU_TEXT = u"<font=4>" + localText.getText("TXT_KEY_PEDIA_SCREEN_TOP", ()).upper() + "</font>"
		
		self.szCategoryTech = localText.getText("TXT_KEY_PEDIA_CATEGORY_TECH", ())		
		self.szCategoryUnit = localText.getText("TXT_KEY_PEDIA_CATEGORY_UNIT", ())		
		self.szCategoryBuilding = localText.getText("TXT_KEY_PEDIA_CATEGORY_BUILDING", ())
		self.szCategoryWonder = localText.getText("TXT_KEY_CONCEPT_WONDERS", ())			
		self.szCategoryBonus = localText.getText("TXT_KEY_PEDIA_CATEGORY_BONUS", ())		
		self.szCategoryTerrain = localText.getText("TXT_KEY_PEDIA_CATEGORY_TERRAIN", ())		
		self.szCategoryFeature = localText.getText("TXT_KEY_PEDIA_CATEGORY_FEATURE", ())		
		self.szCategoryImprovement = localText.getText("TXT_KEY_PEDIA_CATEGORY_IMPROVEMENT", ())		
		self.szCategorySpecialist = localText.getText("TXT_KEY_PEDIA_CATEGORY_SPECIALIST", ())		
		self.szCategoryPromotion = localText.getText("TXT_KEY_PEDIA_CATEGORY_PROMOTION", ())		
		self.szCategoryUnitCombat = localText.getText("TXT_KEY_PEDIA_CATEGORY_UNIT_COMBAT", ())		
		self.szCategoryCiv = localText.getText("TXT_KEY_PEDIA_CATEGORY_CIV", ())		
		self.szCategoryLeader = localText.getText("TXT_KEY_PEDIA_CATEGORY_LEADER", ())		
		self.szCategoryReligion = localText.getText("TXT_KEY_PEDIA_CATEGORY_RELIGION", ())		
		self.szCategoryCivic = localText.getText("TXT_KEY_PEDIA_CATEGORY_CIVIC", ())		
		self.szCategoryProject = localText.getText("TXT_KEY_PEDIA_CATEGORY_PROJECT", ())		
		self.szCategoryConcept = localText.getText("TXT_KEY_PEDIA_CATEGORY_CONCEPT", ())
		self.szCategoryHints = localText.getText("TXT_KEY_PEDIA_CATEGORY_HINTS", ())
		self.szCategoryUpgrades = localText.getText("TXT_KEY_PEDIA_CATEGORY_UPGRADES", ())
		self.szCategoryPromotionTree = localText.getText("TXT_KEY_PEDIA_CATEGORY_PROMOTION_TREE", ())
		self.szCategoryIndex = localText.getText("TXT_KEY_PEDIA_CATEGORY_INDEX", ())

		self.listCategories = [ self.szCategoryTech, 
					self.szCategoryUnit,
					self.szCategoryBuilding,
					self.szCategoryWonder,
					self.szCategoryTerrain, 
					self.szCategoryFeature, 
					self.szCategoryBonus, 
					self.szCategoryImprovement, 
					self.szCategorySpecialist, 
					self.szCategoryPromotion, 
					self.szCategoryUnitCombat,
					self.szCategoryCiv, 
					self.szCategoryLeader,
					self.szCategoryReligion, 
					self.szCategoryCivic, 
					self.szCategoryProject,
					self.szCategoryConcept,
					self.szCategoryHints,
					self.szCategoryUpgrades,
					self.szCategoryPromotionTree,
					self.szCategoryIndex,
		]
								
		# Create a new screen
		screen = self.getScreen()
		screen.setRenderInterfaceOnly(True);
		screen.setScreenGroup(1)
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, False)
		
		# Set background
		screen.addDDSGFC(self.BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("SCREEN_BG_OPAQUE").getPath(), 0, 0, self.W_SCREEN, self.H_SCREEN, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.addPanel(self.TOP_PANEL_ID, u"", u"", True, False, 0, 0, self.W_SCREEN, 55, PanelStyles.PANEL_STYLE_TOPBAR )
		screen.addPanel(self.BOTTOM_PANEL_ID, u"", u"", True, False, 0, 713, self.W_SCREEN, 55, PanelStyles.PANEL_STYLE_BOTTOMBAR )
		screen.setDimensions(screen.centerX(0), screen.centerY(0), self.W_SCREEN, self.H_SCREEN)
		
		# Exit button
		screen.setText(self.EXIT_ID, "Background", self.EXIT_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.X_EXIT, self.Y_EXIT, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1)

		# Back
		screen.setText(self.BACK_ID, "Background", self.BACK_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.X_BACK, self.Y_BACK, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_BACK, 1, -1)
			
		# Forward
		screen.setText(self.NEXT_ID, "Background", self.FORWARD_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.X_FORWARD, self.Y_FORWARD, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_FORWARD, 1, -1)

		# List on the LEFT!
		screen.addListBoxGFC(self.LIST_ID, "", self.X_LINKS, self.Y_LINKS, self.W_LINKS, self.H_LINKS, TableStyles.TABLE_STYLE_STANDARD)
		screen.enableSelect(self.LIST_ID, True)
		screen.setStyle(self.LIST_ID, "Table_StandardCiv_Style")

		# Sublist next to it
		screen.addListBoxGFC(self.SUBLIST_ID, "", self.X_ITEMS_PANE, self.Y_ITEMS_PANE, self.W_ITEMS_PANE, self.H_ITEMS_PANE, TableStyles.TABLE_STYLE_STANDARD)
		screen.enableSelect(self.SUBLIST_ID, True)
		screen.setStyle(self.SUBLIST_ID, "Table_StandardCiv_Style")

		self.createDictionaries()

		
	# Screen construction function
	# Only called on click to FAR LEFT column (LIST_ID)
	def showScreen(self, iCategory):
		CvUtil.pyPrint("showScreen(iCategory=<%i>)"%(iCategory))
		self.iCategory = iCategory

		self.deleteAllWidgets()						
							
		screen = self.getScreen()
		
		bNotActive = (not screen.isActive())
		if bNotActive:
			self.setPediaCommonWidgets()

		# Header...
		szHeader = u"<font=4b>" +localText.getText("TXT_KEY_WIDGET_HELP", ()).upper() + u"</font>"
		szHeaderId = self.getNextWidgetName()
		screen.setLabel(szHeaderId, "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.X_SCREEN, self.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_DESCRIPTION, -1, -1)

		self.panelName = self.getNextWidgetName()
		# I can't just disable this panel.
		# Just 0 size it.
		screen.addPanel(self.panelName, "", "", true, true,
			self.X_PEDIA_PAGE, self.Y_PEDIA_PAGE, 0, self.H_PEDIA_PAGE, PanelStyles.PANEL_STYLE_MAIN)

		
		if self.iLastScreen != CvScreenEnums.PEDIA_MAIN or bNotActive:
			self.iLastScreen = CvScreenEnums.PEDIA_MAIN

		self.placeLinks(iCategory)
		
		# Note to self: this is a deceptively important function here.  It actually calls the 
		# subroutines placeUnits, placeTech, etc based on the hash self.mapCategories.
		# The hash returns the function name, which is subsequently called by the command itself.
		# Tricky, but cool.

		if (self.mapCategories.has_key(iCategory)):
			self.mapCategories.get(iCategory)(-1)

	def isShowAnimationsEnabled(self):
		return gc.getDefineINT("PEDIA_ENABLE_ANIMATIONS") > 0

	def isSkipBlankPage(self):
		return gc.getDefineINT("PEDIA_SKIP_BLANK_PAGE") > 0


	def placeTechs(self, iSubCategory):

		tList = self.pediaTechScreen.getSortedList( gc.getNumTechInfos(), gc.getTechInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_TECH, gc.getTechInfo, iSubCategory)	
	
	
	def placeUnits(self, iSubCategory):

		tList = self.pediaUnitScreen.getSortedList( gc.getNumUnitInfos(), gc.getUnitInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT, gc.getUnitInfo, iSubCategory)	

	
	def placeBuildings(self, iSubCategory):

		tList = self.pediaBuildingScreen.getBuildingSortedList(False)
	
		#tList = self.getSortedList( gc.getNumBuildingInfos(), gc.getBuildingInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, gc.getBuildingInfo, iSubCategory)


	def placeWonders(self, iSubCategory):

		tList = self.pediaBuildingScreen.getBuildingSortedList(True)

		#tList = self.getSortedList( gc.getNumBuildingInfos(), gc.getBuildingInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, gc.getBuildingInfo, iSubCategory)	

	
	def placeBoni(self, iSubCategory):

		tList = self.pediaBonus.getSortedList( gc.getNumBonusInfos(), gc.getBonusInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BONUS, gc.getBonusInfo, iSubCategory)

	
	def placeImprovements(self, iSubCategory):

		tList = self.pediaImprovement.getSortedList( gc.getNumImprovementInfos(), gc.getImprovementInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_IMPROVEMENT, gc.getImprovementInfo, iSubCategory)
			

	def placePromotions(self, iSubCategory):

		tList = self.pediaPromotionScreen.getSortedList( gc.getNumPromotionInfos(), gc.getPromotionInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROMOTION, gc.getPromotionInfo, iSubCategory)
												

	def placeUnitGroups(self, iSubCategory):

		tList = self.pediaUnitChart.getSortedList( gc.getNumUnitCombatInfos(), gc.getUnitCombatInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT_COMBAT, gc.getUnitCombatInfo, iSubCategory)
									

	def placeCivs(self, iSubCategory):

		tList = self.pediaCivilization.getSortedList( gc.getNumCivilizationInfos(), gc.getCivilizationInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIV, gc.getCivilizationInfo, iSubCategory)


	def placeLeaders(self, iSubCategory):
	
		tList = self.pediaLeader.getSortedList( gc.getNumLeaderHeadInfos(), gc.getLeaderHeadInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_LEADER, gc.getLeaderHeadInfo, iSubCategory)

			
	def placeReligions(self, iSubCategory):

		tList = self.pediaReligion.getSortedList( gc.getNumReligionInfos(), gc.getReligionInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_RELIGION, gc.getReligionInfo, iSubCategory)

							
	def placeCivics(self, iSubCategory):

		tList = self.pediaCivic.getSortedList( gc.getNumCivicInfos(), gc.getCivicInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIVIC, gc.getCivicInfo, iSubCategory)

		
	def placeProjects(self, iSubCategory):

		tList = self.pediaProjectScreen.getSortedList( gc.getNumProjectInfos(), gc.getProjectInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROJECT, gc.getProjectInfo, iSubCategory)

	def placeTerrains(self, iSubCategory):

		tList = self.pediaTerrain.getSortedList( gc.getNumTerrainInfos(), gc.getTerrainInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_TERRAIN, gc.getTerrainInfo, iSubCategory)


	def placeFeatures(self, iSubCategory):

		tList = self.pediaFeature.getSortedList( gc.getNumFeatureInfos(), gc.getFeatureInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_FEATURE, gc.getFeatureInfo, iSubCategory)

							
	def placeSpecialists(self, iSubCategory):

		tList = self.pediaSpecialist.getSortedList( gc.getNumSpecialistInfos(), gc.getSpecialistInfo )
		self.displayTopics(tList, WidgetTypes.WIDGET_PEDIA_JUMP_TO_SPECIALIST, gc.getSpecialistInfo, iSubCategory)


	def displayTopics(self, tList, widgeyType, buttonType, iSubCategory):
		CvUtil.pyPrint("displayTopics(tList=<%s>, widgeyType=<%s>, buttonType=<%s>)"%(tList, widgeyType, buttonType))

		screen = self.getScreen()
		
		screen.addListBoxGFC(self.SUBLIST_ID, "", self.X_ITEMS_PANE, self.Y_ITEMS_PANE, self.W_ITEMS_PANE, self.H_ITEMS_PANE, TableStyles.TABLE_STYLE_STANDARD)
		screen.enableSelect(self.SUBLIST_ID, True)
		screen.setStyle(self.SUBLIST_ID, "Table_StandardCiv_Style")

		link = -1
		for i in range(len(tList)):
			item = tList[i]
			screen.appendListBoxString(self.SUBLIST_ID, u"<font=3>" + item[0] + u"</font>", widgeyType, item[1] + self.REFRESH_FLAG, 0, CvUtil.FONT_LEFT_JUSTIFY )
			if item[1] == iSubCategory:
				link = i
		if (link != -1):
			screen.setSelectedListBoxStringGFC(self.SUBLIST_ID, link)
		elif self.isSkipBlankPage() and iSubCategory == -1:
			iScreen = self.iCategory + CvScreenEnums.PEDIA_START
			if self.iCategory == CivilopediaPageTypes.CIVILOPEDIA_PAGE_WONDER:
				iScreen = CivilopediaPageTypes.CIVILOPEDIA_PAGE_BUILDING + CvScreenEnums.PEDIA_START
			self.pediaJump(iScreen, tList[0][1] + self.REFRESH_FLAG, False, False)

		'''
		# This section is supposed to display the icons in the list. It is intended to replace the previous lines.
		# It works by creating a table pretending to be the list box defined above.
		# It is commented out because while it can be made to work,
		# no one has managed to get rid of all the bugs involving selection and refresh.
		
		nColumns = 1
		screen.addTableControlGFC(self.SUBLIST_ID, nColumns, self.X_ITEMS_PANE, self.Y_ITEMS_PANE, self.W_ITEMS_PANE, self.H_ITEMS_PANE, False, False, 16, 16, TableStyles.TABLE_STYLE_STANDARD);
		screen.enableSelect(self.SUBLIST_ID, True)
		screen.setStyle(self.SUBLIST_ID, "Table_StandardCiv_Style")

		for i in range(nColumns):
			screen.setTableColumnHeader(self.SUBLIST_ID, i, "", self.W_ITEMS_PANE/nColumns)
		
		iCounter = 0
		iNumRows = 0
		for item in tList:

			# This line disables the popup text on 'pedia scroll over...hopefully it works
			
			iColumn = iCounter % nColumns
			iRow = iCounter // nColumns
			if iRow >= iNumRows:
				iNumRows += 1
				screen.appendTableRow(self.SUBLIST_ID)
			screen.setTableText(self.SUBLIST_ID, iColumn, iRow, u"<font=3>" + item[0] + u"</font>", buttonType(item[1]).getButton(), widgeyType, item[1], 1, CvUtil.FONT_LEFT_JUSTIFY)
			iCounter += 1
		'''


	def placeConcepts(self, iSubCategory):

		tList = self.pediaHistorical.getSortedList( gc.getNumConceptInfos(), gc.getConceptInfo )
		CvUtil.pyPrint("placeConcepts(tList=<%s>, iSubCategory=<%i>)"%(tList, iSubCategory))
		
		screen = self.getScreen()
		screen.clearListBoxGFC(self.SUBLIST_ID)

		screen.addListBoxGFC( self.SUBLIST_ID, "", self.X_ITEMS_PANE, self.Y_ITEMS_PANE, self.W_ITEMS_PANE, self.H_ITEMS_PANE, TableStyles.TABLE_STYLE_STANDARD ) 
		screen.enableSelect(self.SUBLIST_ID, True)
		screen.setStyle(self.SUBLIST_ID, "Table_StandardCiv_Style")
		
		link = -1
		for i in range(len(tList)):
			topic = tList[i]
			# Not sure why, but when using _NO_HELP, the topic[1] and PageType have to be flipped. Painful...
			screen.appendListBoxString(self.SUBLIST_ID, topic[0], WidgetTypes.WIDGET_PEDIA_DESCRIPTION_NO_HELP, CivilopediaPageTypes.CIVILOPEDIA_PAGE_CONCEPT, topic[1] + self.REFRESH_FLAG, CvUtil.FONT_LEFT_JUSTIFY)
			if (topic[1] == iSubCategory):
				link = i
		if (link != -1):
			screen.setSelectedListBoxStringGFC(self.SUBLIST_ID, link)
		elif self.isSkipBlankPage():
			self.pediaJumpToHistory(CivilopediaPageTypes.CIVILOPEDIA_PAGE_CONCEPT, tList[0][1] + self.REFRESH_FLAG, False, False)

						
	def placeHints(self, iSubCategory):
		screen = self.getScreen()

		screen.deleteWidget("PediaSubList")

		# Lay down some Blooo
		#self.blooPanelName = self.getNextWidgetName()
		#screen.addPanel(self.blooPanelName, "", "", true, true,
		#	self.X_ITEMS_PANE, self.Y_ITEMS_PANE, self.W_ITEMS_PANE + self.W_PEDIA_PAGE-4, self.H_PEDIA_PAGE, PanelStyles.PANEL_STYLE_MAIN)

		self.szHints = self.getNextWidgetName()
		screen.addListBoxGFC( self.szHints, "",
                                      self.X_ITEMS_PANE+20, self.Y_ITEMS_PANE, self.W_ITEMS_PANE + self.W_PEDIA_PAGE-40, self.H_ITEMS_PANE, TableStyles.TABLE_STYLE_STANDARD )
		
		szHintsText = CyGameTextMgr().buildHintsList()
		hintText = string.split( szHintsText, "\n" )
		for hint in hintText:
			if len( hint ) != 0:
				screen.appendListBoxString( self.szHints, hint, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )


	def placeUpgrades(self, iSubCategory):
		screen = self.getScreen()

		# Strangely, this command works:
		self.getScreen().deleteWidget("PediaSubList")

		self.UPGRADES_LIST = self.getNextWidgetName()

		screen.addScrollPanel( self.UPGRADES_LIST, u"", self.X_ITEMS_PANE, self.Y_PEDIA_PAGE, self.W_PEDIA_PAGE + self.W_ITEMS_PANE, self.H_PEDIA_PAGE-20, PanelStyles.PANEL_STYLE_STANDARD )
		screen.setActivation( self.UPGRADES_LIST, ActivationTypes.ACTIVATE_NORMAL )
		
		upgradesGraph = UnitUpgradesGraph.UnitUpgradesGraph(self)
		upgradesGraph.getGraph()
		upgradesGraph.drawGraph()
		
		self.iLastScreen = CvScreenEnums.PEDIA_UPGRADES

	def placePromotionTree(self, iSubCategory):
		screen = self.getScreen()
		
		# Strangely, this command works:
		self.getScreen().deleteWidget("PediaSubList")

		self.PROMOTIONS_LIST = self.getNextWidgetName()
		
		screen.addScrollPanel( self.PROMOTIONS_LIST, u"", self.X_ITEMS_PANE, self.Y_PEDIA_PAGE, self.W_PEDIA_PAGE + self.W_ITEMS_PANE, self.H_PEDIA_PAGE-20, PanelStyles.PANEL_STYLE_STANDARD )
		screen.setActivation( self.PROMOTIONS_LIST, ActivationTypes.ACTIVATE_NORMAL )
		
		promotionsGraph = UnitUpgradesGraph.PromotionsGraph(self)
		promotionsGraph.getGraph()
		promotionsGraph.drawGraph()
		
		self.iLastScreen = CvScreenEnums.PEDIA_PROMOTION_TREE
						
	def placeIndex(self, iSubCategory):

		self.iLastScreen = CvScreenEnums.PEDIA_INDEX
		self.deleteAllWidgets()
		self.pediaIndex.placeIndex(iSubCategory)

										
	def placeLinks(self, iCategory):
		CvUtil.pyPrint("placeLinks()")
		link = 0
		screen = self.getScreen()
		
		screen.clearListBoxGFC(self.LIST_ID)
	
		for item in self.linkList:
			# Note: This was originally intended to place spacers between subjects...
			# I'm not using it, but I'll leave the code for now just in case I change my mind.
			if item[0] == -1:
				screen.appendListBoxString(self.LIST_ID, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
			else:
				lChar = self.linkListGraphics[item[2]]		
				screen.appendListBoxString(self.LIST_ID, lChar + self.listCategories[item[1]], WidgetTypes.WIDGET_PEDIA_MAIN, item[1], 0, CvUtil.FONT_LEFT_JUSTIFY )
				if item[1] == iCategory:
					link = item[0]

		screen.setSelectedListBoxStringGFC(self.LIST_ID, link)

					
	# returns a unique ID for a widget in this screen
	def getNextWidgetName(self):
		szName = self.WIDGET_ID + str(self.nWidgetCount)
		self.nWidgetCount += 1
		return szName
		
	def pediaJump(self, iScreen, iEntry, bRemoveFwdList, bAppendHistory=True):
		if (iEntry < 0):
			return

		CvUtil.pyPrint("pediaJump(iScreen=<%i>, iEntry=<%i>, bRemoveFwdList=<%s>, bAppendHistory=<%s>)"%(iScreen, iEntry, bRemoveFwdList, bAppendHistory))

		self.iActivePlayer = gc.getGame().getActivePlayer()

		if (bRemoveFwdList):
			self.pediaFuture = []

		#this first test is an ugly hack that gets around not being able to add new widget jump targets.
		#The A-Z sublist in the index uses WIDGET_PEDIA_MAIN to send/receive events.
		if (iScreen == CvScreenEnums.PEDIA_MAIN and iEntry >= self.INDEX_FLAG):
			if bAppendHistory:
				self.pediaHistory.append((iScreen, iEntry))
			self.placeIndex(iEntry)
			return
		
		# This is another, similar ugly hack.
		# The C++ (not in SDK) forces us to send all data we want for this event through a single integer (iEntry).
		# Here it is overloaded again to indicate when a forced refresh of the subcategory list is not necessary.
		if iEntry < self.REFRESH_FLAG:
			self.iLastScreen = -1
		else:
			iEntry = iEntry - self.REFRESH_FLAG
		
		if bAppendHistory:
			self.pediaHistory.append((iScreen, iEntry))
		
		if (iScreen == CvScreenEnums.PEDIA_MAIN):
			self.showScreen(iEntry)
		
		jumpToScreen = self.getPediaScreen(iScreen)
		if jumpToScreen != 0:
			jumpToScreen.interfaceScreen(iEntry)

	
	# Special alternative version of pediaJump for PEDIA_HISTORY
	# This is used when called through widget type WIDGET_PEDIA_DESCRIPTION_NO_HELP
	# Not called from links, back, forward, etc.
	def pediaJumpToHistory(self, iScreen, iEntry, bRemoveFwdList, bAppendHistory=True):
		if (iScreen < 0) or (iEntry < 0):
			return

		CvUtil.pyPrint("pediaJumpToHistory(iScreen=<%i>, iEntry=<%i>)"%(iScreen, iEntry))

		self.iActivePlayer = gc.getGame().getActivePlayer()

		if iEntry < self.REFRESH_FLAG:
			self.iLastScreen = -1
		else:
			iEntry = iEntry - self.REFRESH_FLAG

		iEntryId = self.pediaHistorical.getIdFromEntryInfo(iScreen, iEntry)
		if bAppendHistory:
			self.pediaHistory.append((CvScreenEnums.PEDIA_HISTORY, iEntryId))
		if bRemoveFwdList:
			self.pediaFuture = []
		self.pediaHistorical.interfaceScreen(iEntryId)
	

	def back(self):
		print "pedia back"
		if (len(self.pediaHistory) > 1):
			self.pediaFuture.append(self.pediaHistory.pop())
			current = self.pediaHistory.pop()
			self.pediaJump(current[0], current[1], False)
		return 1
		
	def forward(self):
		print "pedia fwd"
		if (self.pediaFuture):
			current = self.pediaFuture.pop()
			self.pediaJump(current[0], current[1], False)
		return 1
		
	def pediaShow(self):
		CvUtil.pyPrint("pediaShow()")
		if (not self.pediaHistory):
			self.pediaHistory.append((CvScreenEnums.PEDIA_MAIN, 0))
			
		current = self.pediaHistory.pop()
		
		# erase history so it doesn't grow too large during the game
		self.pediaFuture = []
		self.pediaHistory = []
		
		# jump to the last screen that was up
		self.pediaJump(current[0], current[1], False)
		
	def link(self, szLink):
		CvUtil.pyPrint("link(szLink=<%s>)"%(szLink))

		if (szLink == "PEDIA_MAIN_TECH"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_TECH), True)			
		if (szLink == "PEDIA_MAIN_UNIT"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_UNIT), True)			
		if (szLink == "PEDIA_MAIN_BUILDING"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_BUILDING), True)			
		if (szLink == "PEDIA_MAIN_TERRAIN"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_TERRAIN), True)			
		if (szLink == "PEDIA_MAIN_FEATURE"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_FEATURE), True)			
		if (szLink == "PEDIA_MAIN_BONUS"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_BONUS), True)			
		if (szLink == "PEDIA_MAIN_IMPROVEMENT"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_IMPROVEMENT), True)			
		if (szLink == "PEDIA_MAIN_SPECIALIST"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_SPECIALIST), True)			
		if (szLink == "PEDIA_MAIN_PROMOTION"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_PROMOTION), True)			
		if (szLink == "PEDIA_MAIN_UNIT_GROUP"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_UNIT_GROUP), True)			
		if (szLink == "PEDIA_MAIN_CIV"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_CIV), True)			
		if (szLink == "PEDIA_MAIN_LEADER"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_LEADER), True)			
		if (szLink == "PEDIA_MAIN_RELIGION"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_RELIGION), True)			
		if (szLink == "PEDIA_MAIN_CIVIC"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_CIVIC), True)			
		if (szLink == "PEDIA_MAIN_PROJECT"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_PROJECT), True)			
		if (szLink == "PEDIA_MAIN_CONCEPT"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_CONCEPT), True)			
		if (szLink == "PEDIA_MAIN_HINTS"):
			return self.pediaJump(CvScreenEnums.PEDIA_MAIN, int(CivilopediaPageTypes.CIVILOPEDIA_PAGE_HINTS), True)

		for i in range(gc.getNumConceptInfos()):
			if (gc.getConceptInfo(i).isMatchForLink(szLink, False)):
				iEntryId = self.pediaHistorical.getIdFromEntryInfo(CivilopediaPageTypes.CIVILOPEDIA_PAGE_CONCEPT, i)
				return self.pediaJump(CvScreenEnums.PEDIA_HISTORY, iEntryId, True)
		for i in range(gc.getNumTechInfos()):
			if (gc.getTechInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_TECH, i, True)
		for i in range(gc.getNumUnitInfos()):
			if (gc.getUnitInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_UNIT, i, True)
		for i in range(gc.getNumBuildingInfos()):
			if (gc.getBuildingInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_BUILDING, i, True)
		for i in range(gc.getNumPromotionInfos()):
			if (gc.getPromotionInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_PROMOTION, i, True)
		for i in range(gc.getNumUnitCombatInfos()):
			if (gc.getUnitCombatInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_UNIT_CHART, i, True)				
		for i in range(gc.getNumBonusInfos()):
			if (gc.getBonusInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_BONUS, i, True)				
		for i in range(gc.getNumTerrainInfos()):
			if (gc.getTerrainInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_TERRAIN, i, True)
		for i in range(gc.getNumFeatureInfos()):
			if (gc.getFeatureInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_FEATURE, i, True)				
		for i in range(gc.getNumImprovementInfos()):
			if (gc.getImprovementInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_IMPROVEMENT, i, True)
		for i in range(gc.getNumCivicInfos()):
			if (gc.getCivicInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_CIVIC, i, True)
		for i in range(gc.getNumCivilizationInfos()):
			if (gc.getCivilizationInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_CIVILIZATION, i, True)
		for i in range(gc.getNumLeaderHeadInfos()):
			if (gc.getLeaderHeadInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_LEADER, i, True)
		for i in range(gc.getNumSpecialistInfos()):
			if (gc.getSpecialistInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_SPECIALIST, i, True)
		for i in range(gc.getNumProjectInfos()):
			if (gc.getProjectInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_PROJECT, i, True)
		for i in range(gc.getNumReligionInfos()):
			if (gc.getReligionInfo(i).isMatchForLink(szLink, False)):
				return self.pediaJump(CvScreenEnums.PEDIA_RELIGION, i, True)
																
	def deleteAllWidgets(self):
		screen = self.getScreen()
		iNumWidgets = self.nWidgetCount
		self.nWidgetCount = 0
		for i in range(iNumWidgets):
			screen.deleteWidget(self.getNextWidgetName())
		self.nWidgetCount = 0
		
	def getPediaScreen(self, eScreen):

		if (eScreen == CvScreenEnums.PEDIA_TECH):
			return self.pediaTechScreen
		if (eScreen == CvScreenEnums.PEDIA_UNIT):
			return self.pediaUnitScreen
		if (eScreen == CvScreenEnums.PEDIA_BUILDING):
			return self.pediaBuildingScreen
		if (eScreen == CvScreenEnums.PEDIA_PROMOTION):
			return self.pediaPromotionScreen
		if (eScreen == CvScreenEnums.PEDIA_UNIT_CHART):
			return self.pediaUnitChart
		if (eScreen == CvScreenEnums.PEDIA_BONUS):
			return self.pediaBonus
		if (eScreen == CvScreenEnums.PEDIA_TERRAIN):
			return self.pediaTerrain
		if (eScreen == CvScreenEnums.PEDIA_FEATURE):
			return self.pediaFeature
		if (eScreen == CvScreenEnums.PEDIA_IMPROVEMENT):
			return self.pediaImprovement
		if (eScreen == CvScreenEnums.PEDIA_CIVIC):
			return self.pediaCivic
		if (eScreen == CvScreenEnums.PEDIA_CIVILIZATION):
			return self.pediaCivilization
		if (eScreen == CvScreenEnums.PEDIA_LEADER):
			return self.pediaLeader
		if (eScreen == CvScreenEnums.PEDIA_SPECIALIST):
			return self.pediaSpecialist
		if (eScreen == CvScreenEnums.PEDIA_PROJECT):
			return self.pediaProjectScreen
		if (eScreen == CvScreenEnums.PEDIA_RELIGION):
			return self.pediaReligion
		if (eScreen == CvScreenEnums.PEDIA_HISTORY):
			return self.pediaHistorical
		
		return 0
			
	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		CvUtil.pyPrint("CvPediaMain: handleInput(inputClass=<%r>, functionName=<%s>, pythonFile=<%s>)"%(inputClass, inputClass.getFunctionName(), inputClass.getPythonFile()))

		if (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CHARACTER):
			if (inputClass.getData() == int(InputTypes.KB_RETURN) or inputClass.getData() == int(InputTypes.KB_ESCAPE)):
				self.getScreen().hideScreen()
				return 1

		inputScreen = self.getPediaScreen(self.iLastScreen)
		if inputScreen != 0:
			return inputScreen.handleInput(inputClass)
		
		return 0