## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
# Sid Meier's Civilization 4
# Copyright Firaxis Games 2005
#
# CvScreen - Base class for all of the Screens

from CvPythonExtensions import *
import CvUtil
import CvScreen

# globals
gc = CyGlobalContext()

class CvPediaScreen( CvScreen.CvScreen ):
	"Civilopedia Base Screen"

	def __init__(self):
		return

	def getSortedList( self, numInfos, getInfo ):
		' returned a list of infos sorted alphabetically '

		if self.isShowGraphicalOnly() or self.isHideIdentical():
			return self.getSortedListFiltered(numInfos, getInfo)
		else:
			return self.getSortedListAll(numInfos, getInfo)

	def getSortedListAll( self, numInfos, getInfo ):
		' returned a list of infos sorted alphabetically '
		infoList = [(0,0)] * numInfos
		for i in range( numInfos ):
			infoList[i] = (self.getTitle(getInfo(i), i), i)
		infoList.sort()
		return infoList

	def getSortedListFiltered( self, numInfos, getInfo ):
		' returned a list of infos sorted alphabetically '

		# count the items we are going to display
		iNumNonGraphical = 0
		for i in range(numInfos):
			if not self.isFiltered(getInfo(i), i):
				iNumNonGraphical += 1

		infoList = [(0,0)] * iNumNonGraphical
		j = 0
		for i in range( numInfos ):
			if not self.isFiltered(getInfo(i), i):
				infoList[j] = (self.getTitle(getInfo(i), i), i)
				j += 1

		infoList.sort()

		return infoList
	
	def isFiltered( self, info, i ):
		' Determine if this should be included in the list. '
		
		if self.isShowGraphicalOnly():
			if self.isGraphicalOnly(info, i):
				return True
		if self.isHideIdentical():
			if self.isIdentical(info, i):
				return True
		return False

	def isGraphicalOnly( self, info, i ):
		' Determine whether this is graphical only. Can be overridden. '
		
		return info.isGraphicalOnly()
	
	def isIdentical( self, info, i ):
		' Determine if this is identical. Overridden for unique units/buildings '
	
		return False
	
	def getTitle( self, info, i ):
		' What to use as the title. Overriden for unique units/buildings '
		
		return info.getDescription()
	
	def isHideIdentical(self):
		return gc.getDefineINT("PEDIA_HIDE_IDENTICAL") > 0
	
	def isShowCivNameForUnique(self):
		return gc.getDefineINT("PEDIA_SHOW_CIV_NAME_FOR_UNIQUE") > 0

	def isShowGraphicalOnly(self):
		return gc.getDefineINT("PEDIA_SHOW_GRAPHICAL_ONLY") > 0
	
	"""
	# This is just an example to show anyone making a meta mod how to make these options dynamic.
	# Of course, you would need to write up the UI yourself.
	def setShowGraphicalOnly(self, bShow):
		gc.setDefineINT("PEDIA_SHOW_GRAPHICAL_ONLY", int(bShow))
	"""