"""
ShowInfo.py
Part of: jrayShowInfo-v208a (tech-trade/resource-trade/open-borders/peace/etc alerts)
         jrayUGH-v208a (jray's Unaltered Gameplay Hodgepodge)
Based on: ShowInfo(enen)-v04 ("[PYTHON] ShowInfo", http://forums.civfanatics.com/showthread.php?t=164589)
Replaces: nothing
Compatible with: Warlords v2.08

Changes to ShowInfo(enen)-v04:
- added support for Show_Available, Show_Unavailable, and Show_Sellable options in Tech_Trade and Resource_Trade INI sections (previously all were treated as True)
- added new Show_Unsellable option to Resource_Trade INI section
- added new GetTechText function to generalize tech info
- shortened many text strings
"""

#---------------------------------------------------------------
# BEG jrayShowInfo-v208a
#{------------------------------
## Copyright (c) 2006, eN.

## ShowInfo: The core info display functionality.
## This file is part of ShowInfo Civilization IV info utility.
## Version: 0.3

from CvPythonExtensions import *
import PyHelpers
import CvCameraControls

import CvConfigParser

from OpponentData import *

gc = CyGlobalContext()

''' sort options
'''
SORTBY_GOLD = 0
SORTBY_GPT = 1
SORTBY_MAP_TRADE = 2
SORTBY_OPEN_BORDER_TRADE = 3
SORTBY_NUM_TECHS_TO_BUY = 4
SORTBY_NUM_TECHS_TO_SELL = 5
SORTBY_NUM_RESOURCES_TO_BUY = 6
SORTBY_NUM_RESOURCES_TO_SELL = 7

# sorters: cmpGold, cmpGpt, cmpMaptrade, cmpOpenborderTrade, cmpAvailTechCount, cmpSellTechCount,
#   cmpUnvailTechCount, cmpAvailResCount, cmpSellResCount, cmpUnavailResCount

''' sort option strings
'''
sortbys = { "GOLD":cmpGold, "GPT":cmpGpt, "MAP":cmpMaptrade, "OPEN_BORDER":cmpOpenborderTrade,
            "TECHS_TO_BUY":cmpAvailTechCount, "TECHS_TO_SELL":cmpSellTechCount,
            "TECHS_CANT_BUY":cmpUnvailTechCount, "RES_TO_BUY":cmpAvailResCount,
            "RES_TO_SELL":cmpSellResCount, "RES_CANT_BUY":cmpUnavailResCount }

''' activation key modifiers
'''
SHIFT_KEY = 1
ALT_KEY = 2
CTRL_KEY = 4

class ShowInfo:

    def __init__(self, eventManager):
        eventManager.addEventHandler("kbdEvent", self.onKbdEvent)
        eventManager.addEventHandler("BeginPlayerTurn", self.onBeginPlayerTurn)
        eventManager.addEventHandler("OnLoad", self.onLoadGame)
        eventManager.addEventHandler("GameStart", self.onGameStart)

        self.eventManager = eventManager
        
        cfgp = CvConfigParser.CvConfigParser("jrayShowInfo.ini")
        
        # gold (money) settings
        #
        # set to True for check for gold and gold per turn trade        
        self.chk_gold = cfgp.getboolean("Gold", "Enabled", True)
        # diplay only GPT values higher or equal than this
        self.min_gpt = cfgp.getint("Gold", "MIN_GPT", 3)
        # diplay only gold values higher or equal than this
        self.min_gold = cfgp.getint("Gold", "MIN_Gold", 10)
        self.gold_increase = cfgp.getboolean("Gold", "Alert_4_Changes_Only", False)

        # trade possiblities
        #
        # set to True to check trade possibilities (individual item check can be turned on/off
        #   with subsequent values)
        self.chk_trade = cfgp.getboolean("General", "Check_Trade", True)
        # set to True to check which opponent consider you as his/her worst enemy
        self.chk_worstenemy = cfgp.getboolean("General", "Check_Worstenemy", True)
        # set to True to check whether an opponent with trade possibilities (have contact and
        #   can trade tech (alphabet)) is researching the same tech
        self.chk_research = cfgp.getboolean("General", "Check_Paralel_Research", True)
        # set to True to check if trading maps is possible
        self.chk_maptrade = cfgp.getboolean("General", "Check_Map", True)
        # set to True to check if open border agreement is possible
        self.chk_openborders = cfgp.getboolean("General", "Check_Open_Borders", True)
        # set to True to check if AI is willing to make peace (in case of war)
        self.chk_peace = cfgp.getboolean("General", "Check_Peace", True)

        # technology trade
        #
        # set to True to check for tech trade: AI willing to sell, AI don't want to sell,
        #   AI willing to buy, AI don't want to buy
        self.chk_techtrade = cfgp.getboolean("Tech_Trade", "Enabled", True)
        # show the techs that can be bought and the opponent is willing to sell it
        self.show_avail_tech = cfgp.getboolean("Tech_Trade", "Show_Available", True)
        # show the techs that can be bought but the opponent is unwilling to sell it
        self.show_unavail_tech = cfgp.getboolean("Tech_Trade", "Show_Unavailable", True)
        # show the techs that the opponent can buy it
        self.show_sellable_tech = cfgp.getboolean("Tech_Trade", "Show_Sellable", True)

        # set to True to check all techs regardless of their trading possibility (debug)
        self.chk_techtrade_all = False

        # resource trade
        #
        # set to True to print the player's name before trade possibilities
        self.add_player_name = cfgp.getboolean("General", "Print_Playername", True)
        # set to True to check for resource trade: AI willing to sell, AI don't want to sell,
        #   AI willing to buy
        self.chk_restrade = cfgp.getboolean("Resource_Trade", "Enabled", True)
        # show the resources that can be bought and the opponent is willing to sell it
        self.show_avail_res = cfgp.getboolean("Resource_Trade", "Show_Available", True)
        # show the resources that can be bought but the opponent is unwilling to sell it
        self.show_unavail_res = cfgp.getboolean("Resource_Trade", "Show_Unavailable", True)
        # show the resources that the opponent can buy it
        self.show_sellable_res = cfgp.getboolean("Resource_Trade", "Show_Sellable", True)
#{beg jrayShowInfo-v208a
        # show the resources that the opponent can but won't buy
        self.show_unsellable_res = cfgp.getboolean("Resource_Trade", "Show_Unsellable", True)
#}end jrayShowInfo-v208a
        
        # output settings
        #
        # set to True to display each trade option in the player's color
        self.use_player_color = cfgp.getboolean("Output", "Use_Player_Color", True)
        # set to True to display map, trade, open borders, gold symbols instead of their names
        self.use_symbols = cfgp.getboolean("Output", "Display_Symbols", True)
        # set to True to display the resource's symbol instead of its name
        self.use_res_symbols = cfgp.getboolean("Output", "Display_Resource_Symbols", True)
        # set to True to print separate messages per opponent
        self.separate_messages = cfgp.getboolean("Output", "Separate_Messages", False)
        # set to display only the first n messages(0 for all). messages can be sorted based
        #   on gold, gpt, nr of tehcs for sale, etc. 
        self.max_enemyinfo = cfgp.getint("Output", "Max_Opponents_to_Show", 4)
        # sorting criteria. use the predefined values!
        self.max_enemyinfo_sorter = []
        if( cfgp.getboolean("Output", "Enable_Sort", True) ):
            s = cfgp.get("Output", "Sort_Fields", "TECHS_TO_BUY, GOLD, GPT, RES_TO_BUY, RES_TO_SELL").upper()
            sby = s.split(",")
            for i in sby:
                k = i.strip()
                if( k in sortbys ):
                    self.max_enemyinfo_sorter.append(sortbys[k])
            
        # true means descending sort
        self.max_enemyinfo_sorter_desc = cfgp.getboolean("Output", "Sort_Desc", True)
        
        # keyboard settings
        #
        # key value 
        self.invoke_key = InputTypes.KB_I
        # key modifier. can be SHIFT_KEY, ALT_KEY or CTRL_KEY.
        self.invoke_key_modifier = SHIFT_KEY
        
        # for debugging purposes. use carefully since it may result in large output
        self.chk_all = False

        self.odc = OpponentDataCollection()
        
    def getInvokeKey(self):
        return int(self.invoke_key)

    def isInvokeKeyModifier_Shift(self):
        return (self.invoke_key_modifier & SHIFT_KEY) != 0

    def isInvokeKeyModifier_Alt(self):
        return (self.invoke_key_modifier & ALT_KEY) != 0

    def isInvokeKeyModifier_Ctrl(self):
        return (self.invoke_key_modifier & CTRL_KEY) != 0
    
    def printInfo(self, iGameTurn, iPlayer):
        if (gc.getPlayer(iPlayer).isHuman()):
            message = u""
            if( self.add_player_name ):
                message += u"Current Player: %s\n" %(gc.getPlayer(iPlayer).getName())
            self.reset()

            for i in range(gc.getMAX_PLAYERS()):
                pl = gc.getPlayer(i)
                if( iPlayer != i and pl.isAlive() and not pl.isBarbarian() and not pl.isMinorCiv() and not pl.isNone() ):
                    if( self.checkWorstEnemy(pl, iPlayer) ):
                        message += u"You are %s's worst enemy!\n" %(pl.getName())

                    if( self.checkCurrentResearch(pl, iPlayer) ):
                        cr = pl.getCurrentResearch()
#{beg ShowInfo-v04
#                         message += u"%s is also researching %s!\n" %(pl.getName(), gc.getTechInfo(cr).getDescription())
#}end ShowInfo-v04
                        if ( cr > 0 and cr < gc.getNumTechInfos() ):
                            message += u"%s is also researching %s!\n" %(pl.getName(), gc.getTechInfo(cr).getDescription())

                    if( self.checkPeace(pl, iPlayer) ):
                        message += u"%s willing to make peace\n" %(pl.getName())
                            
                    if( self.checkTrade(pl, iPlayer) ):
                        tmsg = u""
                        maptrade = False
                        obtrade = False

                        gold = self.checkGold(pl, iPlayer)
                        if( gold > 0 ):
                            tmsg += self.getGoldText(gold)
                            
                        gpt = self.checkGPT(pl, iPlayer)
                        if( gpt > 0 ):
                            tmsg += self.getGPTText(gpt)

                        if( self.checkMapTrade(pl, iPlayer) ):
                            maptrade = True
                            tmsg += self.getMapText()
                            
                        if( self.checkOpenBorders(pl, iPlayer) ):
                            obtrade = True
                            tmsg += self.getOpenbordersText()
                            
                        tech = self.getTechTrade(pl, iPlayer)
                        res = self.getResourceTrade(pl, iPlayer)
                        
                        techmsg = u""
                        resmsg = u""
                        for idx in range(3):
#{beg ShowInfo-v04
#                             techmsg = self.addNotEmpty(techmsg, tech[idx*3], tech[idx*3+1])
#                             resmsg = self.addNotEmpty(resmsg, res[idx*3], res[idx*3+1])
#}end ShowInfo-v04
                            techmsg = self.addNotEmpty(techmsg, tech[idx*3], tech[idx*3+1], u"", u"")
                            resmsg = self.addNotEmpty(resmsg, res[idx*3], res[idx*3+1], u"", u"")
                        
#{beg ShowInfo-v04
#                         tmsg = self.addNotEmpty(tmsg, u"techs", techmsg)
#                         tmsg = self.addNotEmpty(tmsg, u"resources", resmsg)
#}end ShowInfo-v04
                        tmsg = self.addNotEmpty(tmsg, u"", techmsg, u"", u"")
                        tmsg = self.addNotEmpty(tmsg, u"", resmsg, u"", u"")

                        if( len(tmsg) > 0 ):
                            if( self.use_player_color ):
                                tstr = u"<color=%d,%d,%d,%d>- %s %c. %s</color>" %(pl.getPlayerTextColorR(), pl.getPlayerTextColorG(), pl.getPlayerTextColorB(), pl.getPlayerTextColorA(), pl.getName(), CyGame().getSymbolID(FontSymbols.TRADE_CHAR), tmsg)
                            else:
                                tstr = u"- %s %c. %s" %(pl.getName(), CyGame().getSymbolID(FontSymbols.TRADE_CHAR), tmsg)
                            self.odc.append(OpponentData(gold, gpt, maptrade, obtrade, tech[2], tech[8], tech[5], res[2], res[8], res[5], tstr))
                
            if( len(self.odc) > 0 ):
                if( self.separate_messages ):
                    # since the messages are shown in top-down order, we have to sort them in reverse order
                    sortorder = not self.max_enemyinfo_sorter_desc
                else:
                    sortorder = self.max_enemyinfo_sorter_desc

                shortdata = self.odc.getN(self.max_enemyinfo, self.max_enemyinfo_sorter, sortorder)

                for i in shortdata:
                    if( self.separate_messages ):
                        self.printMsg(i.getInfoString())
                    else:
                        message = self.addNotEmpty(message, u"", i.getInfoString(), u"\n", u"")
                
            self.printMsg(message)

    def printMsg(self, msg):
        CyInterface().addImmediateMessage(msg, "")
        
    def addNotEmpty(self, stout, sdesc, s, delim1=u", ", delim2=u" "):
        ret = stout
        if( len(s) > 0 ):
            if( len(stout) > 0 ):
                ret +=  delim1
            ret += sdesc
            ret += delim2
            ret += s

        return ret
        
    def checkWorstEnemy(self, pl1, pl2):
        return self.chk_all or (self.chk_worstenemy and pl1.getWorstEnemyName() == gc.getPlayer(pl2).getName())
    
    def checkTrade(self, pl1, pl2):
        t2 = gc.getTeam(gc.getPlayer(pl2).getTeam())
        return self.chk_all or (self.chk_trade and t2.isHasMet(pl1.getTeam()) and pl1.canTradeWith(pl2) and not t2.isAtWar(pl1.getTeam()))

    def checkGold(self, player, pl):
        if( self.chk_all or (self.chk_gold and gc.getTeam(player.getTeam()).isGoldTrading() and self.min_gold <= player.AI_maxGoldTrade(pl)) ):
            return player.AI_maxGoldTrade(pl)
        else:
            return 0

    def checkGPT(self, player, pl):
        if( self.chk_all or (self.chk_gold and gc.getTeam(player.getTeam()).isGoldTrading() and self.min_gpt <= player.AI_maxGoldPerTurnTrade(pl)) ):
            return player.AI_maxGoldPerTurnTrade(pl)
        else:
            return 0
        
    def checkCurrentResearch(self, pl1, pl2):
        t2 = gc.getTeam(gc.getPlayer(pl2).getTeam())
        return (self.chk_all or self.chk_research) and t2.isHasMet(pl1.getTeam()) and pl1.getCurrentResearch() == gc.getPlayer(pl2).getCurrentResearch()

    def canAndWillTrade(self, pl1, pl2, td):
        return pl1.canTradeItem(pl2, td, False) and pl1.getTradeDenial(pl2, td) == DenialTypes.NO_DENIAL
        
    def getTechTrade(self, pl1, pl2):
        ret1a = u""
        ret1a_cnt = 0
        ret1b = u""
        ret1b_cnt = 0
        ret2 = u""
        ret2_cnt = 0
        
        t1 = gc.getTeam(pl1.getTeam())
        t2 = gc.getTeam(gc.getPlayer(pl2).getTeam())

        tradeData = TradeData()
        tradeData.ItemType = TradeableItems.TRADE_TECHNOLOGIES

        if( self.chk_techtrade_all or (self.chk_techtrade and (t1.isTechTrading() or t2.isTechTrading())) ):
            for i in range(gc.getNumTechInfos()):
                tradeData.iData = i
#{beg ShowInfo-v04
#                 if( t1.isHasTech(i) and not t2.isHasTech(i) and gc.getPlayer(pl2).canResearch(i, False) ):
#                     if( self.canAndWillTrade(pl1, pl2, tradeData) ):
#                         ret1a += u"%s " %(gc.getTechInfo(i).getDescription())
#                         ret1a_cnt += 1
#                     else:
#                         ret1b += u"%s " %(gc.getTechInfo(i).getDescription())
#                         ret1b_cnt += 1
# 
#                 if( t2.isHasTech(i) and not t1.isHasTech(i) and pl1.canResearch(i, False) ):
#                     ret2 += u"%s " %(gc.getTechInfo(i).getDescription())
#                     ret2_cnt += 1
#}end ShowInfo-v04
                if( t1.isHasTech(i) and not t2.isHasTech(i) and gc.getPlayer(pl2).canResearch(i, False) ):
                    if( self.canAndWillTrade(pl1, pl2, tradeData) ):
                        if ( self.show_avail_tech ):
                            ret1a += self.getTechText(i)
                            ret1a_cnt += 1
                    else:
                        if ( self.show_unavail_tech ):
                            ret1b += self.getTechText(i)
                            ret1b_cnt += 1

                if( t2.isHasTech(i) and not t1.isHasTech(i) and pl1.canResearch(i, False) ):
                    if ( self.show_sellable_tech ):
                        ret2 += self.getTechText(i)
                        ret2_cnt += 1

#            if( len(ret1a) > 0 ):
#                ret += "can sell: %s, " %(ret1a)
#            if( len(ret1b) > 0 ):
#                ret += "won't sell: %s, " %(ret1b)
#            if( len(ret2) > 0 ):
#                ret += "can buy %s" %(ret2)
                
#{beg ShowInfo-v04
#         return (u"can sell: ", ret1a, ret1a_cnt, u"won't sell: ", ret1b, ret1b_cnt, u"can buy ", ret2, ret2_cnt)
#}end ShowInfo-v04
        return (u" ", ret1a, ret1a_cnt, u"N ", ret1b, ret1b_cnt, u"B ", ret2, ret2_cnt)
    

    def checkMapTrade(self, pl1, pl2):
        tradeData = TradeData()
        tradeData.ItemType = TradeableItems.TRADE_MAPS
        return (self.chk_all or self.chk_maptrade) and self.canAndWillTrade(pl1, pl2, tradeData)

    def checkOpenBorders(self, pl1, pl2):
        tradeData = TradeData()
        tradeData.ItemType = TradeableItems.TRADE_OPEN_BORDERS
        return (self.chk_all or self.chk_openborders) and self.canAndWillTrade(pl1, pl2, tradeData)

    def checkPeace(self, pl1, pl2):
        tradeData = TradeData()
        tradeData.ItemType = TradeableItems.TRADE_PEACE_TREATY
        t2 = gc.getTeam(gc.getPlayer(pl2).getTeam())
        return (self.chk_all or self.chk_peace) and t2.isAtWar(pl1.getTeam()) and self.canAndWillTrade(pl1, pl2, tradeData)

    def getResourceTrade(self, pl1, pl2):
        tradeData = TradeData()
        tradeData.ItemType = TradeableItems.TRADE_RESOURCES
        ret1a = u""
        ret1a_cnt = 0
        ret1b = u""
        ret1b_cnt = 0
        ret2a = u""
        ret2a_cnt = 0
        ret2b = u""
        ret2b_cnt = 0
        
        if( self.chk_all or self.chk_restrade):
            for i in range(gc.getNumBonusInfos()):
                tradeData.iData = i
            
#{beg ShowInfo-v04
#                 if( pl1.canTradeItem(pl2, tradeData, False) ):
#                     if( pl1.getTradeDenial(pl2, tradeData) == DenialTypes.NO_DENIAL):
#                         ret1a += self.getBonusText(i, pl1.getNumTradeableBonuses(i))
#                         ret1a_cnt += 1
#                     else:
#                         ret1b += self.getBonusText(i, pl1.getNumTradeableBonuses(i))
#                         ret1b_cnt += 1
# 
#                 if( gc.getPlayer(pl2).canTradeItem(pl1.getID(), tradeData, False) ):
#                     if( gc.getPlayer(pl2).getTradeDenial(pl1.getID(), tradeData) == DenialTypes.NO_DENIAL):
#                         ret2a += self.getBonusText(i, gc.getPlayer(pl2).getNumTradeableBonuses(i))
#                         ret2a_cnt += 1
#                     else:
#                         ret2b += self.getBonusText(i, gc.getPlayer(pl2).getNumTradeableBonuses(i))
#                         ret2b_cnt += 1
#}end ShowInfo-v04
                if( pl1.canTradeItem(pl2, tradeData, False) ):
                    if( pl1.getTradeDenial(pl2, tradeData) == DenialTypes.NO_DENIAL):
                        if ( self.show_avail_res ):
                            ret1a += self.getBonusText(i, pl1.getNumTradeableBonuses(i))
                            ret1a_cnt += 1
                    else:
                        if ( self.show_unavail_res ):
                            ret1b += self.getBonusText(i, pl1.getNumTradeableBonuses(i))
                            ret1b_cnt += 1

                if( gc.getPlayer(pl2).canTradeItem(pl1.getID(), tradeData, False) ):
                    if( gc.getPlayer(pl2).getTradeDenial(pl1.getID(), tradeData) == DenialTypes.NO_DENIAL):
                        if ( self.show_sellable_res ):
                            ret2a += self.getBonusText(i, gc.getPlayer(pl2).getNumTradeableBonuses(i))
                            ret2a_cnt += 1
                    else:
                        if ( self.show_unsellable_res ):
                            ret2b += self.getBonusText(i, gc.getPlayer(pl2).getNumTradeableBonuses(i))
                            ret2b_cnt += 1

#            if( len(ret1a) > 0 ):
#                ret += "can sell: %s, " %(ret1a)
#            if( len(ret1b) > 0 ):
#                ret += "won't sell: %s, " %(ret1b)
#            if( len(ret2a) > 0 ):
#                ret += "can buy: %s, " %(ret2a)
#            if( len(ret2b) > 0 ):
#                ret += "won't buy: %s" %(ret2b)

#{beg ShowInfo-v04
#         return (u"can sell: ", ret1a, ret1a_cnt, u"won't sell: ", ret1b, ret1b_cnt, u"can buy: ", ret2a, ret2a_cnt, u"won't buy: ", ret2b, ret2b_cnt)
#}end ShowInfo-v04
        return (u" ", ret1a, ret1a_cnt, u"N ", ret1b, ret1b_cnt, u"B ", ret2a, ret2a_cnt, u"NB ", ret2b, ret2b_cnt)

#####

    def getMapText(self):
        if( self.use_symbols ):
            return u"%c " %(CyGame().getSymbolID(FontSymbols.MAP_CHAR))
        else:
#{beg ShowInfo-v04
#             return u"map "
#}end ShowInfo-v04
            return u"M "

    def getOpenbordersText(self):
        if( self.use_symbols ):
            return u"%c " %(CyGame().getSymbolID(FontSymbols.OPEN_BORDERS_CHAR))
        else:
#{beg ShowInfo-v04
#             return u"open borders "
#}end ShowInfo-v04
            return u"OB "

    def getGoldText(self, g):
        if( self.use_symbols ):
#{beg ShowInfo-v04
#             return u"%c: %d " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar(), g)
#}end ShowInfo-v04
            return u"%c:%d " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar(), g)
        else:
#{beg ShowInfo-v04
#             return u"gold: %d " %(g)
#}end ShowInfo-v04
            return u"G:%d " %(g)

    def getGPTText(self, g):
        if( self.use_symbols ):
#{beg ShowInfo-v04
#             return u"%c/turn: %d " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar(), g)
#}end ShowInfo-v04
            return u"%cPT:%d " %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar(), g)
        else:
#{beg ShowInfo-v04
#             return u"GPT: %d " %(g)
#}end ShowInfo-v04
            return u"GPT:%d " %(g)
        
#{beg jrayShowInfo-v208a
    def getTechText(self, techID):
        return u"%s " %(gc.getTechInfo(techID).getDescription())
#}end jrayShowInfo-v208a

    def getBonusText(self, bonusID, cnt):
        if( self.use_res_symbols ):
            return u"%c(%d) " %(gc.getBonusInfo(bonusID).getChar(), cnt)
        else:
            return u"%s(%d) " %(gc.getBonusInfo(bonusID).getDescription(), cnt)
#####
        
    def reset(self):
        self.odc.reset()

    def clear(self):
        self.odc.clear()

    def onKbdEvent(self, argsList):
        eventType,key,mx,my,px,py = argsList
        
        if( eventType == self.eventManager.EventKeyDown ):
            theKey=int(key)
			
            CvCameraControls.g_CameraControls.handleInput( theKey )
						
            if( theKey == self.getInvokeKey() ):
                    if( (self.isInvokeKeyModifier_Shift() and self.eventManager.bShift) or
                        (self.isInvokeKeyModifier_Alt() and self.eventManager.bAlt) or
                        (self.isInvokeKeyModifier_Ctrl() and self.eventManager.bCtrl) ):
                            self.printInfo(CyGlobalContext().getGame().getGameTurn(), CyGlobalContext().getGame().getActivePlayer())
                            return 1         
        return 0
        
    def onBeginPlayerTurn(self, argsList):
        iGameTurn, iPlayer = argsList
        self.printInfo(iGameTurn, iPlayer)
        return 0

    def onLoadGame(self, argsList):
        self.reset()
        return 0

    def onGameStart(self, argsList):
        self.reset()
        return 0
#}------------------------------
# END jrayShowInfo-v208a
#---------------------------------------------------------------
